{*******************************************************}
{                                                       }
{       IBAccess                                        }
{       Database statistics dialog                      }
{                                                       }
{       Author:                                         }
{       Toni Martir                                     }
{       http://www.ibaccess.org                         }
{       e-mail: toni@pala.com                           }
{                                                       }
{       OpenSource license (MPL)                        }
{                                                       }
{*******************************************************}

unit databasestat;

interface

uses
  SysUtils, Classes, QForms, QDialogs,
  diaser, IBServices,rpibdb, ibman, QMask, QExtCtrls, QStdCtrls, QButtons,
  QControls;

type
  TFStatistics = class(TFService)
    IBStat: TIBStatisticalService;
    GRestore: TGroupBox;
    RadioService: TRadioButton;
    RadioCommandline: TRadioButton;
    IbMan1: TIbManager;
    GOptions: TGroupBox;
    CheckIndexpages: TCheckBox;
    CheckSystemRelations: TCheckBox;
    CheckDatapages: TCheckBox;
    CheckDBLog: TCheckBox;
    CheckHeaderPages: TCheckBox;
    GType: TGroupBox;
    RadioLock: TRadioButton;
    RadioStand: TRadioButton;
    GLock: TGroupBox;
    RadioT1: TRadioGroup;
    GAll: TGroupBox;
    RadioT2: TRadioGroup;
    Label4: TLabel;
    Label5: TLabel;
    EInterval: TMaskEdit;
    ESamples: TMaskEdit;
    Checkrecord: TCheckBox;
    procedure BOKClick(Sender: TObject);
    procedure RadioServiceClick(Sender: TObject);
    procedure RadioStandClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure RadioT1Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

procedure ShowstatIBdatabase(var username,password,server,database:string;var protocol:TRpIBXProtocol;dbconf:TRpDBIBconf);

implementation

uses ibproces;

{$R *.xfm}

procedure ShowstatIBdatabase(var username,password,server,database:string;var protocol:TRpIBXProtocol;dbconf:TRpDBIBconf);
var
 dia:TFStatistics;
begin
 dia:=TFStatistics.create(Application);
 try
  dia.Eusername.text:=username;
  dia.EPassword.text:=password;
  dia.EServer.text:=server;
  dia.dbconf:=dbconf;
  dia.EServerChange(dia);
  dia.EBase.text:=Database;
  dia.comboprotocol.itemindex:=integer(protocol);
  dia.Radiocommandline.checked:=ibman.globalusecommandline;
  dia.showmodal;
  if dia.executed then
  begin
   username:=dia.USername;
   password:=dia.Password;
   database:=dia.database;
   server:=dia.Server;
   protocol:=dia.protocol;
  end;
 finally
  dia.free;
 end;
end;

procedure TFStatistics.BOKClick(Sender: TObject);
begin
  inherited;
 // Validate the database
 if RadioCommandline.checked then
 begin
  EServer.text:='';
  ibman1.serverpath:=ibman.globalserverpath;
  ibman1.Username:=Eusername.text;
  ibman1.Password:=EPassword.text;
  Ibman1.filename:=BuildIBDatabaseName((Length(Trim(EServer.text))>0),TRpIBXProtocol(ComboProtocol.ItemIndex),EServer.Text,EBase.text);
  Ibman1.Statoptions:=[];
  if CheckDatapages.checked then
   Ibman1.statoptions:=Ibman1.statoptions+[sDataPages];
  if CheckDBLog.checked then
   Ibman1.statoptions:=Ibman1.statoptions+[sDBlog];
  if CheckHeaderPages.checked then
   Ibman1.statoptions:=Ibman1.statoptions+[sHeaderPages];
  if CheckIndexPages.checked then
   Ibman1.statoptions:=Ibman1.statoptions+[sIndexPages];
  if CheckSystemrelations.checked then
   Ibman1.statoptions:=Ibman1.statoptions+[sSystemrelations];
  if Checkrecord.checked then
   Ibman1.statoptions:=Ibman1.StatOptions+[sAvgRecords];
  IBMan1.LockStatOption:=TIBManLockSOption(RadioT1.ItemIndex);
  IBMan1.LockStat2Option:=TIBManLockS2Option(RadioT2.ItemIndex);
  if RadioLock.Checked then
  begin
   IbMan1.LockSamples:=StrToInt(ESamples.Text);
   IbMan1.LockSampleInterval:=StrToInt(EInterval.Text);
   ibMan1.ShowLockStats;
  end
  else
   IBMan1.ShowStats;
 end
 else
 begin
  IBStat.Params.clear;
  IBStat.params.add('user_name='+EUsername.text);
  IBStat.params.add('password='+EPassword.text);
  IBStat.ServerName:=Trim(EServer.Text);
  IBStat.databasename:=EBase.text;
  if Length(IBStat.ServerName)=0 then
  begin
   IBStat.protocol:=Local;
  end
  else
  begin
   IBStat.protocol:=TProtocol(ComboProtocol.itemindex);
  end;
  IBStat.options:=[];
  if CheckDatapages.checked then
   IBStat.options:=IBStat.options+[DataPages];
  if CheckDBLog.checked then
   IBStat.options:=IBStat.options+[DBLog];
  if CheckHeaderPages.checked then
   IBStat.options:=IBStat.options+[Headerpages];
  if CheckIndexPages.checked then
   IBStat.options:=IBStat.options+[IndexPages];
  if CheckSystemrelations.checked then
   IBStat.options:=IBStat.options+[Systemrelations];

  IBStat.Attach;
  try
   IBStat.ServiceStart;
   ibproces.COntrolIBService(IBStat);
  finally
   try
    IBStat.Detach;
   except
   end;
  end;
 end;
end;

procedure TFStatistics.RadioServiceClick(Sender: TObject);
begin
  inherited;
 EServer.text:='';
 EServerCHange(Self);
 Label6.visible:=RadioService.checked;
 Eserver.visible:=RadioService.checked;
 LProtocol.visible:=RadioService.checked;
 ComboProtocol.visible:=RadioService.checked;
 RadioLock.enabled:=Not RadioService.checked;
 if Not RadioLock.Enabled then
 begin
  RadioStand.Checked:=True;
 end;
 RadioStandCLick(Self);
end;

procedure TFStatistics.RadioStandClick(Sender: TObject);
begin
  inherited;
 GOptions.Visible:=RadioStand.Checked;
 Glock.Visible:=Not RadioStand.Checked;
 EBase.Visible:=RadioStand.Checked;
 EServer.Visible:=RadioStand.Checked  And RadioService.checked;
 ComboProtocol.Visible:=RadioStand.Checked And RadioService.checked;
 Label6.Visible:=RadioStand.Checked  And RadioService.checked;
 Label1.Visible:=RadioStand.Checked;
 LProtocol.Visible:=RadioStand.Checked And RadioService.checked;
 ComboAlias.Visible:=RadioStand.Checked;
 LAlias.Visible:=RadioStand.Checked;
 BConfig.Visible:=RadioStand.Checked;
 BSearch.Visible:=RadioStand.Checked AND (Length(Trim(EServer.text))>0);
end;

procedure TFStatistics.FormCreate(Sender: TObject);
begin
  inherited;
 RadioT1.Itemindex:=0;
 RadioT2.Itemindex:=0;
end;

procedure TFStatistics.RadioT1Click(Sender: TObject);
begin
  inherited;
 GAll.Visible:=RadioT1.Itemindex=3;
end;

end.
