{*******************************************************}
{                                                       }
{       IBAccess                                        }
{       Recompute indexes functions and proces          }
{                                                       }
{       Author:                                         }
{       Toni Martir                                     }
{       http://www.ibaccess.org                         }
{       e-mail: toni@pala.com                           }
{                                                       }
{       OpenSource license (MPL)                        }
{                                                       }
{*******************************************************}

unit ibidxstat;

interface

uses Classes, SysUtils,QForms, ibdatabase,ibsql,
  ibacconst, Db,
  DBClient, QControls, QComCtrls, QStdCtrls;

type
  TFIBIndexStat = class(TForm)
    CancelBtn: TButton;
    Trans: TIBTransaction;
    IBSQL1: TIBSQL;
    Label1: TLabel;
    LIndexcount: TLabel;
    Label2: TLabel;
    LINdexname: TLabel;
    PBar: TProgressBar;
    Label3: TLabel;
    LTablename: TLabel;
    DIndex: TClientdataset;
    DIndexINDEXNAME: TStringField;
    DIndexTABLENAME: TStringField;
    procedure CancelBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    base:TIBDatabase;
    docancel:boolean;
    indexcount,icount:integer;
    tablename,indexname:string;
    procedure AppIdle(Sender:TObject;var done:boolean);
    procedure Checkcancel;
  public
    { Public declarations }
  end;


procedure ibrecomputeindexstat(base:TIBDatabase;table,index:string);

implementation

procedure ibrecomputeindexstat(base:TIBDatabase;table,index:string);
var
 dia:TFIBIndexStat;
 oldonidle:TIdleevent;
begin
 dia:=TFIBIndexStat.create(Application);
 try
  oldonidle:=application.onidle;
  application.onidle:=dia.appidle;
  try
   dia.base:=base;
   dia.trans.defaultdatabase:=base;
   dia.ibsql1.database:=base;
   dia.indexname:=index;
   dia.tablename:=table;
   dia.showmodal;
  finally
   application.onidle:=oldonidle;
  end;
 finally
  dia.free;
 end;
end;

procedure TFIBIndexstat.Checkcancel;
begin
 LIndexcount.caption:=IntTostr(indexcount);
 LIndexname.Caption:=DIndexINDEXNAME.AsString;
 LTablename.caption:=DIndexTABLENAME.Asstring;
 PBar.position:=icount;
 Application.processMessages;
 if docancel then
  Raise Exception.create(SRpOperationAborted);
end;

procedure TFIBIndexStat.AppIdle(Sender:TObject;var done:boolean);
var
 adeli:String;
begin
 application.onidle:=nil;
 adeli:='';
 if base.sqldialect=3 then
  adeli:='"';
 done:=false;
 try
  indexcount:=0;
  if length(indexname)>0 then
  begin
   DIndex.AppendRecord([tablename,indexname]);
   indexcount:=1;
  end
  else
  begin
   trans.StartTransaction;
   try
    icount:=0;
    IBSQL1.SQL.text:='SELECT RDB$RELATION_NAME,RDB$INDEX_NAME FROM RDB$INDICES ';
    if length(tablename)>0 then
     IBSQL1.SQL.text:=IBSQL1.SQL.text+' WHERE RDB$RELATION_NAME='''+tablename+''' ';
    IBSQL1.SQL.text:=IBSQL1.SQL.text+' ORDER BY RDB$RELATION_NAME,RDB$INDEX_NAME';
    IBSQL1.ExecQuery;
    try
     if IBSQL1.Eof then
      Raise Exception.Create(SRpNoindexestorecompute);
     While Not IBSQL1.EOF do
     begin
      Dindex.Appendrecord([IBSQL1.FieldByname('RDB$RELATION_NAME').AsString,
       IBSQL1.FieldByname('RDB$INDEX_NAME').AsString]);
      inc(indexcount);
      Checkcancel;
      IBSQL1.Next;
     end;
    finally
     IBSQL1.close;
    end;
   finally
    trans.rollback;
   end;
  end;
  icount:=0;
  // Recomputes statistics
  trans.StartTransaction;
  try
   PBar.max:=indexcount;
   DIndex.first;
   While Not DIndex.eof do
   begin
    inc(icount);
    CheckCancel;
    IBSQL1.SQL.Text:='SET STATISTICS INDEX '+adeli+DIndexINDEXNAME.AsString+adeli;
    IBSQL1.ExecQuery;
    DIndex.next;
   end;
   trans.commit;
  except
   trans.rollback;
   raise;
  end;
 finally
  close;
 end;
end;

{$R *.xfm}

procedure TFIBIndexStat.CancelBtnClick(Sender: TObject);
begin
 docancel:=true;
end;

procedure TFIBIndexStat.FormCreate(Sender: TObject);
begin
 DIndex.Createdataset;
end;

end.
