{*******************************************************}
{                                                       }
{       ibman                                           }
{       TIbManager component                            }
{                                                       }
{       Author:                                         }
{       Toni Martir                                     }
{       http://www.ibaccess.org                         }
{       e-mail: toni@pala.com                           }
{                                                       }
{       OpenSource license (MPL)                        }
{                                                       }
{*******************************************************}

// service:string
// hostname:service_mgr for TCP/IP
// \\hostname\service_mgr for NetBEUI
// hostname@service_mgr for SPX
// service_mgr for Local services

unit ibman;

interface

uses SysUtils, Classes,
{$IFDEF MSWINDOWS}
  registry,imagehlp,filectrl,
{$ENDIF}
{$IFDEF LINUX}
  libc,
{$ENDIF}
  ibmanst,rpversion,variants,
  QForms,QStdCtrls, QControls;


type
  tpagesizetype=(tp1024,tp2048,tp4096,tp8192,tp16384,tp32768);
  TIBManRestoreoption=(rReplaceexisting,rUseallspace,rDeactivateindexes,
     rCommitEachTable,rReadOnly,rNoshadows,rnovalidate);
  TIBManRestoreoptions=set of TIBManrestoreoption;

  TIBManBackupoption=(bConvertTables,bIgnoreChecksums,bIgnoreLimbo,
    bMetadataonly,bNogarbagecollection,bNonTransportable,bOldMetadata,bExpand);
  TIBManBackupoptions=set of TIBManBackupoption;

  TIBManRepairoption=(reValidate,reIgnoreChecksums,reMenddatabase,reReadOnly,reValidateRecords,reKillShadows,relimbotrans);
  TIBManRepairOptions=set of TIBManRepairoption;

  TIBManStatoption=(sDatapages,sDBLog,sHeaderPages,sIndexPages,sSystemrelations,sAvgrecords);
  TIBManStatOptions=set of TIBManStatoption;

  TIBManLockSoption=(lStatic,lSummary,lOwner,lAll);
  TIBManLockS2Option=(l2All,l2Threads,l2Operations,l2Numbers);

  TShutdownOption=(shForce,shDenyTrans,shDenyCon);

  TLimboAction=(rLimboCommit,rLimboRollback,rLimboRecover);
const
 constpagesizes:array [tp1024..tp32768] of string=('1024','2048','4096','8192','16384','32768');
 type
{$IFDEF MSWINDOWS}
  TThreadCopy=class;
{$ENDIF}

  TFCopyProgressIB = class;
  TIbManager=class(TComponent)
   private
    FServer:string;
    fparams:TStrings;
    FServerPath:string;
    DiaCopy:TFCopyProgressIB;
{$IFDEF MSWINDOWS}
    FCopy:TThreadCopy;
    FBusy:boolean;
{$ENDIF}
    FOldOnIdle:TIdleEvent;
    fbackupfiles,frestorefiles:TStrings;
    FPageSize:TPageSizeType;
    FFilename:string;
    FUsername,FPassword:string;
    FShowProgress:boolean;
    FService:string;
    FRestoreBuffers:integer;
    FRole:string;
    FRestoreOptions:TIBManRestoreoptions;
    FBackupOptions:TIBManBackupoptions;
    FBackBlockingfactor:integer;
    FAutoclose:Boolean;
    FRepairOptions:TIBManRepairOptions;
    FlimboAction:TlimboAction;
    FStatOptions:TIBManStatOptions;
    FLockStatOption:TIBManLockSOption;
    FLockStat2Option:TIBManLockS2Option;
    FLockSampleInterval,FLockSamples:integer;
    FShutdownOption:TShutdownOption;
    FShutdownseconds:integer;
    function GetServerName:string;
{$IFDEF MSWINDOWS}
    procedure CheckBusy;
    function GetBusy:boolean;
    procedure Endcopy(Sender:TObject);
    procedure endsincro;
    procedure silentterminate(Sender:TObject);
    procedure IdleCopy(Sender:TObject;var done:Boolean);
{$ENDIF}
    function IBAction(title:string;closeonexit,forceconsole:boolean;doerror:boolean):boolean;
    procedure Setrestorefiles(Alist:TStrings);
    procedure Setbackupfiles(Alist:TStrings);
   public
    opresult:boolean;
    constructor Create(AOwner:TComponent);override;
    destructor destroy;override;
    function Backup:boolean;
    function Restore:boolean;
    function Sweep:boolean;
    function Repair:boolean;
    function  ShutDown:boolean;
    function  BringOnline:boolean;
    function EnableForcedWrites(Value:Boolean):boolean;
    function SetSweepInterval(Value:integer):boolean;
    function SetCachePages(Value:integer):boolean;
    function AddUser(username,password,firstname,middlename,lastname:string;uid,gid:variant):boolean;
    function ModifyUser(username,password,firstname,middlename,lastname:string;uid,gid:variant):boolean;
    function DeleteUser(username:string):boolean;
    function RepairLimbotransaction(tranid:integer):boolean;
    function RepairLimbotransactions:boolean;
    function ShowStats:boolean;
    function ShowLockStats:boolean;
    function SetSQLDialect(dialect:integer):boolean;
    function SetReadonly(Readonly:boolean):boolean;
    property ServerName:string read GetServerName;
{$IFDEF MSWINDOWS}
    property Busy:boolean read GetBusy;
{$ENDIF}
   published
    property PageSize:tpagesizetype read FPageSize write FPageSize default tp1024;
    property Filename:string read FFIleName write FFileName;
    property Username:string read FuserName write FUsername;
    property Password:string read FPassword write FPassword;
    property Role:string read FRole write FRole;
    property ShowProgress:boolean read FShowProgress write FShowProgress default true;
    property Service:string read FService write FService;
    property RestoreBuffers:integer read FRestoreBuffers write FRestoreBuffers default 0;
    property RestoreOptions:TIBManRestoreoptions read FRestoreOptions write FRestoreOptions;
    property BackupOptions:TIBManBackupoptions read FBackupOptions write FBackupOptions;
    property BackBlockingfactor:integer read FBackBlockingfactor write FBackBlockingfactor default 0;
    property RepairOptions:TIBManRepairOptions read FRepairOptions write FRepairOptions;
    property StatOptions:TIBManStatOptions read FStatOptions write FStatOptions;
    property Autoclose:boolean read FAutoclose write FAutoclose default True;
    property LimboAction:TLimboAction read FLimboAction write FLimboAction default rLimboCommit;
    property LockStatOption:TIBManLockSOption read FLockStatOption write FLockStatOption default lStatic;
    property LockStat2Option:TIBManLockS2Option read FLockStat2Option write FLockStat2Option default l2All;
    property LockSampleInterval:integer read FlockSampleinterval write FLockSampleInterval default 3;
    property LockSamples:integer read FlockSamples write FLockSamples default 0;
    property ShutdownOption:TShutdownOption read FShutdownOption write FShutdownOption default shForce;
    property Shutdownseconds:integer read FShutdownseconds write FShutdownseconds default 1;
    property Serverpath:string read FServerpath write FServerpath;
    property backupfiles:TStrings read fbackupfiles write setbackupfiles;
    property restorefiles:TStrings read frestorefiles write setrestorefiles;
   end;


  TFCopyProgressIB = class(TForm)
    BCancel: TButton;
    LProceso: TLabel;
    LMessages: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure BCancelClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    docancel:boolean;
    doerror:boolean;
{$IFDEF MSWINDOWS}
    fcopy:TThreadCopy;
    cancelat:boolean;
    commandline:string;
{$ENDIF}
    Fib:TIbManager;
    opresult:integer;
{$IFDEF LINUX}
    procedure IdleCopy(Sender:TObject;var done:Boolean);
{$ENDIF}
  public
    { Public declarations }
    params:TStringlist;
  end;

{$IFDEF MSWINDOWS}
 TThreadCopy=class(TThread)
  private
   Fcommandline,FExename:string;
   Form:TFCopyProgressIB;
   FMessage:string;
   opresult:integer;
   forceconsole:boolean;
   procesinfo:TProcessInformation;
  public
   endedwitherror:Boolean;
   endoffile:boolean;
   errormsg:string;
   closeonexit:boolean;
   hpipewrite:THandle;
   hpipemultiple:THandle;
   constructor createthread;
   destructor destroy;override;
   procedure execute;override;
   procedure CShowMessage;
  end;
{$ENDIF}


var
 globalserverpath:string;
 globalusecommandline:boolean;

implementation



{$R *.xfm}


{$IFDEF MSWINDOWS}

procedure TIBManager.silentterminate(Sender:Tobject);
begin
 opresult:=TThreadCopy(Sender).opresult=0;
end;


constructor TThreadCopy.createthread;
begin
 inherited create(true);
end;

destructor TThreadCopy.destroy;
begin
 inherited destroy;
end;

procedure TIbManager.endsincro;
var
 doclose:boolean;
 closeonexit:boolean;
begin
 diacopy.opresult:=fcopy.opresult;
 closeonexit:=FCopy.closeonexit;
 doclose:=False;
 try
  if FCopy.endedwitherror then
  begin
   ShowMessage(SIBCopyError+FCopy.errormsg);
   doclose:=true;
  end
  else
   if diacopy.docancel then
   begin
    ShowMessage(SIBCanceled);
    doclose:=true;
   end;
 finally
  diacopy.Fcopy:=nil;
  diacopy.docancel:=true;
  FCopy:=nil;
  Application.OnIdle:=FOldOnIdle;
  if doclose then
  begin
   diacopy.close;
  end
  else
  begin
   // Set forewindow
   SetForegroundWindow(diacopy.Handle);
   diacopy.BCancel.Caption:=SIBOK;
   if closeonexit then
   begin
    diacopy.Fib.opresult:=true;
    PostMessage(diacopy.Handle,WM_CLOSE,0,0);
   end;
  end;
 end;
end;

procedure TIbManager.EndCopy(Sender:TObject);
begin
 (Sender As TThreadCopy).Synchronize(endsincro);
end;


procedure TIbManager.CheckBusy;
begin
 if Busy then
  Raise Exception.Create(SIBBusy);
end;

function TIbManager.GetBusy:Boolean;
begin
 Result:=False;
 if FCopy<>nil then
  result:=true;
end;

procedure TTHreadCopy.CShowMessage;
begin
 if assigned(Form) then
 begin
  Form.LMessages.Lines.Add(FMessage);
  Form.LMessages.SelStart:=Length(FOrm.Lmessages.text);
  Form.LMessages.SelLength:=0;
  if Endoffile then
   Form.update;
 end;
end;

procedure TThreadCopy.execute;
var
 startinfo:TStartupinfo;
 hpiperead:THandle;
 readcount:integer;
 buffer:Pchar;
 linecount:string;
 tempstring:string;
 index:integer;
 exitcode:DWORD;
 numerror:integer;
begin
 endedwitherror:=false;
 errormsg:='';
 try
 linecount:='';
 buffer:=AllocMem(200);
 try
  if not CreatePipe(hpiperead,hpipewrite,nil,0) then
  Begin
   FMessage:=SIBComunError;
   Synchronize(Cshowmessage);
  end;
  try
   try
    with startinfo do
    begin
     cb:=sizeof(startinfo);
     lpReserved:=nil;
     lpDesktop:=nil;
     lpTitle:=Pchar(SIBBackup);
     dwX:=0;
     dwY:=0;
     dwXSize:=400;
     dwYSize:=400;
     dwXCountChars:=80;
     dwYCountChars:=25;
     dwFillAttribute:=FOREGROUND_RED or BACKGROUND_RED or BACKGROUND_GREEN or BACKGROUND_BLUe;
     dwFlags:=STARTF_USECOUNTCHARS or STARTF_USESTDHANDLES or STARTF_USESHOWWINDOW;
     // On NT Direct pipes not work
     if (IsWindowsNT or forceconsole) then
      dwFlags:=STARTF_USECOUNTCHARS or STARTF_USESHOWWINDOW
     else
     begin
      dwFlags:=STARTF_USECOUNTCHARS or STARTF_USESTDHANDLES or STARTF_USESHOWWINDOW;
     end;
     if Not forceconsole then
     begin
      wShowWindow:=SW_HIDE;
     end;

     cbReserved2:=0;
     lpreserved2:=nil;
     hStdInput:=GetStdhandle(STD_INPUT_HANDLE);
     hStdOutput:=hpipewrite;
     hStdError:=hpipewrite;
    end;
    // Creates the interbase command line proces
    if Not CreateProcess(Pchar(FExename),Pchar(Fcommandline),nil,nil,True,NORMAL_PRIORITY_CLASS or CREATE_NEW_PROCESS_GROUP,nil,nil,
    startinfo,procesinfo) then
     Raise Exception.Create(SIBErrorcreatepr);
   finally
    CloseHandle(hpipewrite);
   end;
   Endoffile:=false;
   FMessage:='';
   linecount:='';
   if (IsWindowsNT or forceconsole) then
   begin
     FMessage:=SIBWaitEnd;
     Synchronize(cshowmessage);
     WaitForSingleObject(procesinfo.hProcess,INFINITE);
   end
   else
   begin
   While not Endoffile do
   begin
    if Not ReadFile(hpiperead,buffer^,1,DWORD(readcount),nil) then
    begin
     numerror:=GetLasterror;
     FMessage:=SysErrorMessage(numerror);
     Synchronize(cshowmessage);
     buffer[readcount]:=chr(0);
     exit;
    end
    else
    begin
     if readcount=0 then
      endoffile:=true;
     buffer[readcount]:=chr(0);
    end;
    // If the buffer have an Intro char...
    tempstring:=strpas(buffer);
    index:=Pos(chr(13),tempstring);
    if index<1 then
    begin
     linecount:=linecount+tempstring;
     if Endoffile then
     begin
      FMessage:=linecount;
      Synchronize(cshowmessage);
     end;
    end
    else
    begin
     FMessage:=linecount+Copy(tempstring,1,index-1);
     While length(FMessage)>0 do
     begin
      if ((FMessage[1]=chr(10)) or (FMessage[1]=chr(13)) or (FMessage[1]=chr(9))) then
       delete(FMessage,1,1)
      else
       break;
     end;
     linecount:=Copy(tempstring,index+1,200);
     While length(linecount)>0 do
     begin
      if ((linecount[1]=chr(10)) or (linecount[1]=chr(13))) then
       delete(linecount,1,1)
      else
       break;
     end;
     Synchronize(cshowmessage);
     FMessage:='';
    end;
   end;
   end;
  finally
   CloseHandle(hpiperead);
  end;
 finally
  Freemem(buffer);
 end;
 except
  on E:Exception do
  begin
   errormsg:=E.Message;
   endedwitherror:=True;
  end;
 end;
 // Waits until the process ends
 WaitForSingleObject(procesinfo.hProcess,INFINITE);
 if NOt GetExitCodeProcess(procesinfo.hProcess,exitcode) then
 begin
  FMessage:=SIBNoResult;
  Synchronize(cshowmessage);
  opresult:=-1;
 end
 else
 begin
  opresult:=exitcode;
  FMessage:=IntToStr(exitcode);
  Synchronize(cshowmessage);
 end;
 if assigned(form) then
 begin
  if form.cancelat then
   opresult:=-1;
 end;
end;

{$ENDIF}

constructor TIbManager.Create(AOwner:TComponent);
begin
 inherited Create(AOwner);
 FFilename:='';
 FServer:='';
 fparams:=TStringlist.Create;
 fbackupfiles:=TStringlist.create;
 frestorefiles:=TStringlist.create;
{$IFDEF MSWINDOWS}
 FCopy:=nil;
 FBusy:=False;
{$ENDIF}
 FpageSize:=tp8192;
 FUsername:='SYSDBA';
 Password:='masterkey';
 FShowProgress:=True;
 FRestorebuffers:=0;
 FRestoreOptions:=[rReplaceexisting];
 FLockSampleInterval:=3;
 FShutdownseconds:=1;
end;

destructor TIbManager.destroy;
begin
 frestorefiles.free;
 fbackupfiles.free;
 fparams.free;
 inherited destroy;
end;


// Forceconsole only affects windows nt
// doStderror redirects stderror output
function TIbManager.IBAction(title:string;closeonexit,forceconsole:boolean;doerror:boolean):boolean;
{$IFDEF MSWINDOWS}
var
 han:integer;
 rec:Tsmallrect;
{$ENDIF}
begin
 Result:=false;
 if fparams.count<1 then
  exit;
{$IFDEF MSWINDOWS}
 // Implementation for windows missing
 // iterate throught params to construct command line
 if FShowProgress then
 begin
  AllocConsole();
  try
  han:=GetStdhandle(STD_OUTPUT_HANDLE);
  if ((Not IsWindowsNT) and (not forceconsole)) then
  begin
   rec.top:=100;
   rec.left:=100;
   rec.Right:=-200;
   rec.bottom:=-200;
   SetConsoleWindowInfo(han,true,rec);
  end;
  // Check if it es doing a operation
  CheckBusy;
  nomserver:=Servername;
  // New progress form
  diacopy:=TFCopyProgressIB.Create(Application);
  try
   diacopy.doerror:=doerror;
   diacopy.caption:=title;
   FCopy:=TThreadCopy.Createthread;
   FCopy.forceconsole:=forceconsole;
   // NT not accept spaces on command line
   FCopy.Fcommandline:={NomServer+}'\'+commandline;
   FCopy.FExename:=NomServer+'\'+exename;
   FCopy.freeonterminate:=true;
   FCopy.Onterminate:=EndCopy;
   FCopy.Form:=diacopy;
   FCopy.closeonexit:=closeonexit;
   diacopy.fcopy:=FCopy;
   diacopy.FIb:=self;
   FOldOnIdle:=Application.Onidle;
   Application.OnIdle:=IdleCopy;
   diacopy.showmodal;
  except
   diacopy.free;
   raise;
  end;
  Result:=(diacopy.opresult=0);
  finally
   FreeConsole();
  end;
 end
 else
 begin
  FCopy:=TThreadCopy.Create(True);
  FCopy.forceconsole:=forceconsole;
  // NT not accept spaces on command line
  nomserver:=Servername;
  opresult:=false;
  FCopy.Fcommandline:={NomServer+}'\'+commandline;
  FCopy.FExename:=NomServer+'\'+exename;
  FCopy.Onterminate:=silentterminate;
  FCopy.freeonterminate:=true;
  FCopy.Resume;
  fcopy.WaitFor;
  Result:=opresult;
 end;
{$ENDIF}
{$IFDEF LINUX}
 // New progress form
 diacopy:=TFCopyProgressIB.Create(Application);
 try
  diacopy.caption:=title;
  // NT not accept spaces on command line
  diacopy.params.assign(fparams);
  diacopy.doerror:=doerror;
  FOldOnIdle:=Application.Onidle;
  Application.OnIdle:=diacopy.IdleCopy;
  diacopy.showmodal;
  Result:=(diacopy.opresult=0);
  Application.Onidle:=FoldOnidle
 finally
  diacopy.free;
 end;
{$ENDIF}
end;

function TIbManager.Backup:boolean;
var
 i:integer;
begin
 fparams.clear;
{$IFDEF MSWINDOWS}
 fparams.add(servername+'\gbak.exe');
{$ENDIF}
{$IFDEF LINUX}
 fparams.add(servername+'/gbak');
{$ENDIF}
 fparams.add('-Z');
 fparams.add('-V');
 fparams.add('-B');

 if bIgnoreChecksums in BackupOptions then
  fparams.add('-IG');
 if Length(Service)>0 then
 begin
  fparams.add('-service');
  fparams.add(service);
 end;
 if Length(Role)>0 then
 begin
  fparams.add('-role');
  fparams.add(role);
 end;
 if bConverttables in BackupOptions then
  fparams.add('-convert');
 if bIgnoreLimbo in BackupOptions then
  fparams.add('-limbo');
 if bNogarbagecollection in BackupOptions then
  fparams.add('-g');
 if bMetadataonly in BackupOptions then
  fparams.add('-metadata');
 if bNonTransportable in BackupOptions then
  fparams.add('-nt');
 if bOldMetadata in BackupOptions then
  fparams.add('-ol');
 if bExpand in BackupOptions then
  fparams.add('-expand');
 if FBackBlockingfactor>0 then
 begin
  fparams.add('-fa');
  fparams.add(IntTostr(FBackBlockingFactor));
 end;
 fparams.add('-user');
 fparams.add(fusername);
 fparams.add('-password');
 fparams.add(fpassword);
 fparams.add(filename);
 for i:=0 to fbackupfiles.count-1 do
  fparams.add(fbackupfiles.strings[i]);
 Result:=IBAction(SIBBackup,autoclose,false,true);
end;

function TIbManager.Restore:boolean;
var
 i:integer;
begin
 fparams.clear;
{$IFDEF MSWINDOWS}
 fparams.add(servername+'\gbak.exe');
{$ENDIF}
{$IFDEF LINUX}
 fparams.add(servername+'/gbak');
{$ENDIF}
 fparams.add('-C');
 fparams.add('-Z');
 fparams.add('-V');

 fparams.add('-user');
 fparams.add(fusername);
 fparams.add('-password');
 fparams.add(fpassword);

 fparams.Add('-p');
 fparams.add(constpagesizes[FPagesize]);
 if rReplaceexisting in RestoreOptions then
  fparams.add('-R');
 if FRestorebuffers>0 then
 begin
  fparams.add('-buffers');
  fparams.add(inttostr(Frestorebuffers));
 end;
 if rNoShadows in RestoreOptions then
  fparams.add('-kill');
 if rDeactivateindexes in RestoreOptions then
  fparams.add('-inactive');
 if rReadOnly in RestoreOptions then
 begin
  fparams.add('-mode');
  fparams.add('read_only');
 end;
 if Length(Service)>0 then
 begin
  fparams.add('-service');
  fparams.add(service);
 end;
 if rCommitEachTable in RestoreOptions then
  fparams.add('-one_at_a_time');
 if rUseAllSpace in RestoreOptions then
  fparams.add('-use_all_space');
 if rNoValidate in RestoreOptions then
  fparams.add('-no_validity');
 for i:=0 to fbackupfiles.count-1 do
  fparams.add(fbackupfiles.strings[i]);
 for i:=0 to frestorefiles.count-1 do
  fparams.add(frestorefiles.strings[i]);
 Result:=IBAction(SIBRestoring,autoclose,false,true);
end;

function TIbManager.Repair:boolean;
begin
 fparams.clear;
{$IFDEF MSWINDOWS}
 fparams.add(servername+'\gfix.exe');
{$ENDIF}
{$IFDEF LINUX}
 fparams.add(servername+'/gfix');
{$ENDIF}

 if reMenddatabase in FRepairOptions then
 begin
  fparams.add('-mend');
  fparams.Add('-ignore');
  fparams.Add('-z');
 end
 else
 if reValidate in FRepairOptions then
 begin
  fparams.add('-v');
  if reIgnorechecksums in FRepairOptions then
   fparams.add('-ignore');
  if reReadonly in FRepairOptions then
   fparams.add('-no_update');
  if reValidateRecords in FRepairOptions then
   fparams.add('-full');
  if reValidateRecords in FRepairOptions then
   fparams.add('-kill');
 end
 else
 if relimbotrans in FRepairOptions then
 begin
  fparams.add('-list');
 end;

 fparams.add('-user');
 fparams.add(fusername);
 fparams.add('-password');
 fparams.add(fpassword);
 fparams.add(filename);
 Result:=IBAction(SIBRepairing,autoclose,false,false);
end;


function TibManager.RepairLimbotransactions:boolean;
begin
 fparams.clear;
{$IFDEF MSWINDOWS}
 fparams.add(servername+'\gfix.exe');
{$ENDIF}
{$IFDEF LINUX}
 fparams.add(servername+'/gfix');
{$ENDIF}
 fparams.Add('-z');

 case limboaction of
  rLimboRollback:
   fparams.add('-rollback');
  rLimboCommit:
   fparams.add('-commit');
  rLimboRecover:
   fparams.add('-two_phase');
 end;
 fparams.add('all');

 fparams.add('-user');
 fparams.add(fusername);
 fparams.add('-password');
 fparams.add(fpassword);
 fparams.add(filename);
 Result:=IBAction(SIBRepairing,autoclose,false,false);
end;

function TIbManager.RepairLimbotransaction(tranid:integer):boolean;
begin
 fparams.clear;
{$IFDEF MSWINDOWS}
 fparams.add(servername+'\gfix.exe');
{$ENDIF}
{$IFDEF LINUX}
 fparams.add(servername+'/gfix');
{$ENDIF}
 fparams.Add('-z');

 case limboaction of
  rLimboRollback:
   fparams.add('-rollback');
  rLimboCommit:
   fparams.add('-commit');
  rLimboRecover:
   fparams.add('-two_phase');
 end;
 fparams.add(inttostr(tranid));

 fparams.add('-user');
 fparams.add(fusername);
 fparams.add('-password');
 fparams.add(fpassword);
 fparams.add(filename);
 Result:=IBAction(SIBRepairing,autoclose,false,false);
end;

function TIbManager.SetSweepInterval(Value:integer):boolean;
begin
 fparams.clear;
{$IFDEF MSWINDOWS}
 fparams.add(servername+'\gfix.exe');
{$ENDIF}
{$IFDEF LINUX}
 fparams.add(servername+'/gfix');
{$ENDIF}
 fparams.Add('-h');
 fparams.Add(IntTostr(Value));
 fparams.add('-user');
 fparams.add(fusername);
 fparams.add('-password');
 fparams.add(fpassword);
 fparams.add(filename);
 Result:=IBAction(SIBSetSweep,autoclose,false,false);
end;

function TIbManager.SetCachePages(Value:integer):boolean;
begin
 fparams.clear;
{$IFDEF MSWINDOWS}
 fparams.add(servername+'\gfix.exe');
{$ENDIF}
{$IFDEF LINUX}
 fparams.add(servername+'/gfix');
{$ENDIF}
 fparams.Add('-buffers');
 fparams.Add(IntTostr(Value));
 fparams.add('-user');
 fparams.add(fusername);
 fparams.add('-password');
 fparams.add(fpassword);
 fparams.add(filename);
 Result:=IBAction(SIBSetCachepages,autoclose,false,false);
end;

function TIbManager.EnableForcedWrites(Value:Boolean):boolean;
var
 amessage:string;
begin
 fparams.clear;
{$IFDEF MSWINDOWS}
 fparams.add(servername+'\gfix.exe');
{$ENDIF}
{$IFDEF LINUX}
 fparams.add(servername+'/gfix');
{$ENDIF}
 fparams.Add('-write');
 if value then
 begin
  amessage:=SIBEnable;
  fparams.add('sync')
 end
 else
 begin
  amessage:=SIBDisable;
  fparams.Add('async');
 end;
 fparams.add('-user');
 fparams.add(fusername);
 fparams.add('-password');
 fparams.add(fpassword);
 fparams.add(filename);
 Result:=IBAction(amessage,autoclose,false,false);
end;

function TIbManager.Sweep:boolean;
begin
 fparams.clear;
{$IFDEF MSWINDOWS}
 fparams.add(servername+'\gfix.exe');
{$ENDIF}
{$IFDEF LINUX}
 fparams.add(servername+'/gfix');
{$ENDIF}
 fparams.Add('-sweep');
 fparams.add('-user');
 fparams.add(fusername);
 fparams.add('-password');
 fparams.add(fpassword);
 fparams.add(filename);
 Result:=IBAction(SIBSweep,autoclose,false,false);
end;


function TIbManager.GetServerName:string;
{$IFDEF MSWINDOWS}
var
 reg:TRegistry;
{$ENDIF}
begin
 if Length(FServer)>0 then
 begin
  result:=FServer;
  Exit;
 end;
 if Length(FServerPath)>0 then
 begin
  Result:=FServerpath;
  exit;
 end;
{$IFDEF MSWINDOWS}
 reg:=TRegistry.create;
 try
  Reg.Access:=KEY_READ;
  Reg.RootKey:=HKEY_LOCAL_MACHINE;
  if Not reg.OpenKey('Software\InterBase Corp\InterBase\CurrentVersion',false) then
   if Not reg.OpenKey('Software\Borland\InterBase\CurrentVersion',false) then
    Raise Exception.Create(SIBNotInstalled);
  Result:=reg.ReadString('ServerDirectory');
  if Length(Result)<=0 then
   Raise Exception.create(SIBNotInstalled);
  if Result[length(Result)]='\' then
   Result:=Copy(Result,1,Length(Result)-1);
 finally
  reg.free;
 end;
{$ENDIF}
{$IFDEF LINUX}
 Result:='/opt/interbase/bin';
{$ENDIF}

end;

procedure TFCopyProgressIB.FormCreate(Sender: TObject);
begin
 docancel:=false;
 LProceso.Caption:=SIBProgress;
 BCancel.Caption:=SIBCancel;
{$IFDEF LINUX}
 params:=TStringlist.create;
{$ENDIF}
end;

procedure TFCopyProgressIB.BCancelClick(Sender: TObject);
begin
 docancel:=true;
 if BCancel.Caption=SIBOK then
 begin
  if FIb<>nil then
   FIb.opresult:=True;
  close;
  exit;
 end
{$IFDEF MSWINDOWS}
  // Try to end the interbase command line process
  if FCopy<>nil then
  begin
   docancel:=true;
   // This for Win95
   if not GenerateConsoleCtrlEvent(CTRL_C_EVENT,FCopy.procesinfo.dwProcessId) then
    RaiseLastWin32Error;
   // This for WinNT
   GenerateConsoleCtrlEvent(CTRL_BREAK_EVENT,FCopy.procesinfo.dwProcessId);
   cancelat:=true;
   opresult:=-1;
   fcopy.opresult:=-1;
  end
  else
   close;
{$ENDIF}
end;


procedure TFCopyProgressIB.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
 Action:=cafree;
{$IFDEF LINUX}
 params.free;
{$ENDIF}
end;

procedure TFCopyProgressIB.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
 canclose:=docancel;
end;



procedure TFCopyProgressIB.FormShow(Sender: TObject);
begin
 if assigned(FIb) then
  FIb.opresult:=False;
end;


function TIbManager.ShutDown:boolean;
begin
 fparams.clear;
{$IFDEF MSWINDOWS}
 fparams.add(servername+'\gfix.exe');
{$ENDIF}
{$IFDEF LINUX}
 fparams.add(servername+'/gfix');
{$ENDIF}
 fparams.Add('-shut');
 case FShutdownOption of
  shForce:
   fparams.Add('-force');
  shDenyCon:
   fparams.Add('-attach');
  shDenyTrans:
   fparams.Add('-tran ');
 end;
 fparams.add(IntToStr(Shutdownseconds));
 fparams.Add('-z');
 fparams.add('-user');
 fparams.add(fusername);
 fparams.add('-password');
 fparams.add(fpassword);
 fparams.add(filename);
 Result:=IBAction(SIBShutdown,autoclose,false,false);
end;

function TIbManager.BringOnline:boolean;
begin
 fparams.clear;
{$IFDEF MSWINDOWS}
 fparams.add(servername+'\gfix.exe');
{$ENDIF}
{$IFDEF LINUX}
 fparams.add(servername+'/gfix');
{$ENDIF}
 fparams.Add('-z');
 fparams.add('-user');
 fparams.add(fusername);
 fparams.add('-password');
 fparams.add(fpassword);
 fparams.Add('-online');
 fparams.add(filename);
 Result:=IBAction(SIBBringOnLine,autoclose,false,false);
end;


function TIbManager.SetSQLDialect(dialect:integer):boolean;
begin
 fparams.clear;
{$IFDEF MSWINDOWS}
 fparams.add(servername+'\gfix.exe');
{$ENDIF}
{$IFDEF LINUX}
 fparams.add(servername+'/gfix');
{$ENDIF}
 fparams.Add('-z');
 fparams.add('-user');
 fparams.add(fusername);
 fparams.add('-password');
 fparams.add(fpassword);
 fparams.Add('-sql_dialect');
 fparams.Add(inttostr(dialect));
 fparams.add(filename);
 Result:=IBAction(SIBDialect,autoclose,false,false);
end;

function TIbManager.SetReadonly(Readonly:boolean):boolean;
begin
 fparams.clear;
{$IFDEF MSWINDOWS}
 fparams.add(servername+'\gfix.exe');
{$ENDIF}
{$IFDEF LINUX}
 fparams.add(servername+'/gfix');
{$ENDIF}
 fparams.Add('-z');
 fparams.add('-user');
 fparams.add(fusername);
 fparams.add('-password');
 fparams.add(fpassword);
 fparams.Add('-mode');
 if readonly then
  fparams.Add('read_only')
 else
  fparams.add('read_write');
 fparams.add(filename);
 Result:=IBAction(SIBReadonly,autoclose,false,false);
end;


function TIbManager.AddUser(username,password,firstname,middlename,lastname:string;uid,gid:variant):boolean;
begin
 fparams.clear;
{$IFDEF MSWINDOWS}
 fparams.add(servername+'\gsec.exe');
{$ENDIF}
{$IFDEF LINUX}
 fparams.add(servername+'/gssec');
{$ENDIF}
 fparams.add('-user');
 fparams.add(fusername);
 fparams.add('-password');
 fparams.add(fpassword);
 fparams.add('-mo');
 fparams.add(username);
 fparams.add('-pw');
 if length(password)>0 then
  fparams.add(password)
 else
  fparams.Add('');
 fparams.add('-fname');
 if length(firstname)>0 then
  fparams.add(firstname)
 else
  fparams.add('');
 fparams.add('-mname');
 if length(middlename)>0 then
  fparams.add(middlename)
 else
  fparams.add('');
 fparams.add('-lname');
 if length(lastname)>0 then
  fparams.add(lastname)
 else
  fparams.add('');
 fparams.add('-uid');
 if uid=null then
  fparams.add('0')
 else
  fparams.add(IntToStr(integer(uid)));
 fparams.add('-gid');
 if gid=null then
  fparams.add('0')
 else
  fparams.add(IntToStr(integer(gid)));
 if length(filename)>0 then
 begin
  fparams.Add('-database');
  fparams.add(filename);
 end;
 Result:=IBAction(SIBNewUser,autoclose,false,false);
end;

function TIbManager.ModifyUser(username,password,firstname,middlename,lastname:string;uid,gid:variant):boolean;
begin
 fparams.clear;
{$IFDEF MSWINDOWS}
 fparams.add(servername+'\gsec.exe');
{$ENDIF}
{$IFDEF LINUX}
 fparams.add(servername+'/gssec');
{$ENDIF}
 fparams.add('-user');
 fparams.add(fusername);
 fparams.add('-password');
 fparams.add(fpassword);
 fparams.add('-mo');
 fparams.add(username);
 fparams.add('-pw');
 if length(password)>0 then
  fparams.add(password)
 else
  fparams.Add('');
 fparams.add('-fname');
 if length(firstname)>0 then
  fparams.add(firstname)
 else
  fparams.add('');
 fparams.add('-mname');
 if length(middlename)>0 then
  fparams.add(middlename)
 else
  fparams.add('');
 fparams.add('-lname');
 if length(lastname)>0 then
  fparams.add(lastname)
 else
  fparams.add('');
 fparams.add('-uid');
 if uid=null then
  fparams.add('0')
 else
  fparams.add(IntToStr(integer(uid)));
 fparams.add('-gid');
 if gid=null then
  fparams.add('0')
 else
  fparams.add(IntToStr(integer(gid)));
 if length(filename)>0 then
 begin
  fparams.Add('-database');
  fparams.add(filename);
 end;
 Result:=IBAction(SIBModifyuser,autoclose,false,false);
end;

function TIbManager.DeleteUser(username:string):boolean;
begin
 fparams.clear;
{$IFDEF MSWINDOWS}
 fparams.add(servername+'\gsec.exe');
{$ENDIF}
{$IFDEF LINUX}
 fparams.add(servername+'/gssec');
{$ENDIF}
 fparams.add('-user');
 fparams.add(fusername);
 fparams.add('-password');
 fparams.add(fpassword);
 fparams.add('-de');
 fparams.add(username);
 if length(filename)>0 then
 begin
  fparams.Add('-database');
  fparams.add(filename);
 end;
 Result:=IBAction(SIBDeleteuser,autoclose,false,false);
end;

function TIbManager.ShowStats:boolean;
begin
 fparams.clear;
{$IFDEF MSWINDOWS}
 fparams.add(servername+'\gstat.exe');
{$ENDIF}
{$IFDEF LINUX}
 fparams.add(servername+'/gstat');
{$ENDIF}
 fparams.add('-Z');
 if sDatapages in FStatOptions then
  fparams.add('-data');
 if sDBLog in FStatOptions then
  fparams.add('-log');
 if sHeaderpages in FStatOptions then
  fparams.add('-header');
 if sIndexpages in FStatOptions then
  fparams.add('-index');
 if sSystemrelations in FStatOptions then
  fparams.add('-system');
 if sAvgrecords in FStatOptions then
  fparams.add('-r');
 fparams.add('-user');
 fparams.add(fusername);
 fparams.add('-password');
 fparams.add(fpassword);
 fparams.add(filename);
 Result:=IBAction(SIBStatistics,autoclose,false,false);
end;

function TIbManager.ShowLockStats:boolean;
begin
 fparams.clear;
{$IFDEF MSWINDOWS}
 fparams.add(servername+'\iblockpr.exe');
{$ENDIF}
{$IFDEF LINUX}
 fparams.add(servername+'/gds_lock_mgr');
{$ENDIF}
 case LockStatOption of
  lStatic:
   fparams.add('-a');
  lSummary:
   fparams.add('-o');
  lOwner:
   fparams.add('-w');
  lAll:
   begin
    case LockStat2Option of
     l2All:
      fparams.add('-i');
     l2Threads:
      fparams.add('-ia');
     l2Operations:
      fparams.add('-io');
     l2Numbers:
      fparams.add('-iw');
    end;
    if LockSamples>0 then
    begin
     fparams.add(IntToStr(LockSampleInterval));
     fparams.add(IntToStr(LockSamples));
    end;
   end;
 end;
 Result:=IBAction(SIBStatistics,autoclose,false,false);
end;


procedure TibManager.Setrestorefiles(Alist:tStrings);
begin
 frestorefiles.assign(alist);
end;

procedure TIbmanager.Setbackupfiles(Alist:TStrings);
begin
 fbackupfiles.assign(alist);
end;

{$IFDEF MSWINDOWS}
procedure TIbManager.IdleCopy(Sender:TObject;var done:Boolean);
begin
 Done:=True;
 Application.OnIdle:=nil;
 FCopy.Resume;
end;
{$ENDIF}

{$IFDEF LINUX}
procedure TFCopyProgressIB.IdleCopy(Sender:TObject;var done:Boolean);
var
 child:__pid_t;
 apipe:array [0..1] of integer;
 pint:PInteger;
 i:integer;
 theparams:array [0..500] of pchar;
 buffer:array [0..1] of char;
 pbuf:Pchar;
 readed:integer;
 astring:string;
begin
 Done:=True;
 Application.OnIdle:=nil;

 if params.count>255 then
  Raise exception.create(SIBTooManyParams);
 pint:=@apipe;
 pbuf:=@buffer;
 for i:=0 to params.count-1 do
 begin
  theparams[i]:=Pchar(params[i]);
 end;
 theparams[params.count]:=nil;
 if (-1=pipe(pint)) then
  Raise Exception.create(SIBErrorCreatePipe);
 child:=fork;
 if child=-1 then
  Raise Exception.Create(SIBErrorFork);
 if child<>0 then
 begin
  __close(apipe[0]);
  if doerror then
  begin
   dup2(apipe[1],2);
  end
  else
  begin
   dup2(apipe[1],2);
   dup2(apipe[1],1);
  end;
  // The child executes the command
  execvp(theparams[0],PPChar(@theparams))
 end
 else
 begin
  __close(apipe[1]);
  try
   astring:='';
   buffer[1]:=chr(0);
   repeat
    readed:=__read(apipe[0],pbuf^,1);
    if readed>0 then
    begin
     if pbuf[0]=chr(10) then
     begin
      LMessages.Lines.Add(astring);
      astring:='';
     end
     else
      astring:=astring+pbuf[0];
    end;
    Application.processmessages;
    if docancel then
     break;
   until readed=0;
   BCancel.caption:=SIBOK;
   if length(astring)>0 then
    LMessages.Lines.Add(astring);
  finally
   __close(apipe[0]);
  end;
 end;
end;
{$ENDIF}


initialization

globalserverpath:='';
globalusecommandline:=false;

end.
