{*******************************************************}
{                                                       }
{       IBPack                                          }
{       Parameter definition, and assignement           }
{                                                       }
{       Author:                                         }
{       Toni Martir                                     }
{       http://www.ibaccess.org                         }
{       e-mail: toni@pala.com                           }
{                                                       }
{       OpenSource license (MPL)                        }
{                                                       }
{*******************************************************}

unit rpparamdef;

interface

uses Classes, QForms, QControls, QStdCtrls,
  Db, IBCustomDataSet, IBQuery,SysUtils,
 ibsql, QDialogs, QMask, QButtons,variants;

type
  TFparamdef = class(TForm)
    BOK: TButton;
    CancelBtn: TButton;
    Label1: TLabel;
    LParams: TListBox;
    LType: TLabel;
    ComboType: TComboBox;
    LValue: TLabel;
    LDescrip: TLabel;
    EDescrip: TEdit;
    EValue: TMaskEdit;
    DValue: TMaskEdit;
    TValue: TMaskEdit;
    BLoad: TBitBtn;
    CheckNull: TCheckBox;
    OpenDialog1: TOpenDialog;
    procedure LParamsClick(Sender: TObject);
    procedure EValueChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ComboTypeClick(Sender: TObject);
    procedure CheckNullClick(Sender: TObject);
    procedure BLoadClick(Sender: TObject);
    procedure DValueChange(Sender: TObject);
    procedure TValueChange(Sender: TObject);
  private
    { Private declarations }
    fparams:TParams;
    procedure SetParams(Value:TParams);
  public
    { Public declarations }
    property params:TParams read fparams write Setparams;
  end;

function ChangeParamValues(IBQuery1:TIBQuery):boolean;overload;
function ChangeParamValues(IBSQL1:TIBSQL):boolean;overload;

implementation

function ChangeParamValues(IBQuery1:TIBQuery):boolean;
var
 dia:TFParamdef;
begin
 Result:=false;
 if IBQuery1.ParamCount<1 then
  exit;
 dia:=TFParamdef.Create(Application);
 try
  dia.params:=IBQuery1.params;
  dia.LDescrip.visible:=false;
  dia.EDescrip.visible:=false;
  dia.LParams.itemindex:=0;
  dia.lparamsclick(dia.lparams);
  Result:=dia.showmodal=mrok;
  if result then
  begin
   IBQuery1.Params.AssignValues(dia.params);
  end;
 finally
  dia.free;
 end;
end;

function ChangeParamValues(IBSQL1:TIBSQL):boolean;
var
 ibquery1:TIBQuery;
 i:integer;
begin
 ibquery1:=TIBQuery.Create(Application);
 try
  ibquery1.sql.text:=IBSQL1.sql.text;
  for i:=0 to IBSQL1.params.count-1 do
  begin
   ibquery1.Params.Items[i].Value:=IBSQL1.Params.Vars[i].Value;
  end;
  Result:=ChangeParamvalues(IBQuery1);
  for i:=0 to IBQUery1.params.count-1 do
  begin
   IBSQL1.Params.Vars[i].Value:=ibquery1.Params.Items[i].Value;
  end;
 finally
  ibquery1.free;
 end;
end;

{$R *.xfm}

procedure TFparamdef.SetParams(Value: TParams);
var
 i:integer;
 alist:TStringlist;
begin
 fparams.assign(Value);
 alist:=TStringlist.create;
 try
  for i:=0 to params.Count-1 do
  begin
   alist.add(params.items[i].Name);
  end;
  LParams.items.Assign(alist);
 finally
  alist.free;
 end;
end;


procedure TFparamdef.LParamsClick(Sender: TObject);
var
 param:TParam;
begin
 // Set the param type and value
 param:=params.items[lparams.itemindex];
 if param.isnull then
 begin
  CheckNull.checked:=true;
  ComboType.Itemindex:=0;
  EValue.visible:=false;
  DValue.visible:=false;
  TValue.visible:=false;
  BLoad.visible:=false;
  LType.visible:=false;
  combotype.visible:=false;
  LValue.visible:=false;
  exit;
 end;
 CheckNull.onclick:=nil;
 try
  Checknull.checked:=false;
 finally
  Checknull.OnClick:=CheckNullClick;
 end;
 LType.visible:=true;
 combotype.visible:=true;
 LValue.visible:=true;
 case param.DataType of
  ftUnknown:
   begin
    ComboType.Itemindex:=0;
    EValue.visible:=false;
    DValue.visible:=false;
    TValue.visible:=false;
    BLoad.visible:=false;
   end;
  ftString, ftBytes, ftVarBytes,ftFixedChar, ftWideString:
   begin
    ComboType.Itemindex:=7;
    DValue.visible:=false;
    TValue.visible:=false;
    BLoad.visible:=false;
    EValue.visible:=True;
    if not param.IsNull then
     EValue.text:=param.AsString;
   end;
  ftSmallint, ftInteger, ftWord:
   begin
    ComboType.Itemindex:=1;
    DValue.visible:=false;
    TValue.visible:=false;
    BLoad.visible:=false;
    EValue.visible:=True;
    if not param.IsNull then
     EValue.TExt:=param.AsString;
   end;
  ftLargeint:
   begin
    ComboType.Itemindex:=1;
    DValue.visible:=false;
    TValue.visible:=false;
    BLoad.visible:=false;
    EValue.visible:=True;
    if not param.IsNull then
     EValue.Text:=param.AsString;
   end;
//  ftBoolean:
  ftFloat:
   begin
    ComboType.Itemindex:=1;
    DValue.visible:=false;
    TValue.visible:=false;
    BLoad.visible:=false;
    EValue.visible:=True;
    if not param.IsNull then
     EValue.Text:=param.AsString;
   end;
  ftCurrency, ftBCD:
   begin
    ComboType.Itemindex:=1;
    DValue.visible:=false;
    TValue.visible:=false;
    BLoad.visible:=false;
    EValue.visible:=True;
    if not param.IsNull then
     EValue.Text:=param.AsString;
   end;
  ftDate:
   begin
    ComboType.Itemindex:=4;
    DValue.visible:=True;
    TValue.visible:=false;
    BLoad.visible:=false;
    EValue.visible:=false;
    if not param.IsNull then
     DValue.Text:=param.AsString;
   end;
  ftTime:
   begin
    ComboType.Itemindex:=5;
    DValue.visible:=False;
    TValue.left:=DValue.left;
    TValue.visible:=True;
    BLoad.visible:=false;
    EValue.visible:=false;
    if not param.IsNull then
     TValue.Text:=param.AsString;
   end;
  ftDateTime:
   begin
    ComboType.Itemindex:=6;
    TValue.left:=DValue.left+DValue.width+4;
    DValue.visible:=True;
    TValue.visible:=True;
    BLoad.visible:=false;
    EValue.visible:=false;
    if not param.IsNull then
    begin
     DValue.Text:=FormatDatetime('dd/mm/yyyy',param.AsDate);
     TValue.Text:=FormatDatetime('hh:mm:ss',param.AsTime);
    end;
   end;
  ftBlob, ftMemo,ftGraphic, ftFmtMemo,ftParadoxOle,ftDBaseOle, ftTypedBinary:
   begin
    ComboType.Itemindex:=8;
    DValue.visible:=false;
    TValue.visible:=false;
    BLoad.visible:=True;
    EValue.visible:=False;
   end
   else
   begin
    ComboType.Itemindex:=0;
    EValue.visible:=false;
    DValue.visible:=false;
    TValue.visible:=false;
    BLoad.visible:=false;
   end;
 end;
end;

procedure TFparamdef.EValueChange(Sender: TObject);
var
 param:tparam;
begin
 param:=params.items[lparams.itemindex];
 param.AsString:=EValue.text;
{ param:=params.items[lparams.itemindex];
 case EValue.EditType of
  tegeneral:
   begin
   end;
  teinteger:
   begin
    try
     param.AsInteger:=EValue.AsInteger;
    except
     param.AsInteger:=0;
    end;
   end;
  teLargeint:
   begin
    try
     param.AsCurrency:=Trunc(EValue.AsCurrency);
    except
     param.AsCurrency:=0;
    end;
   end;
  tefloat:
   begin
    try
     param.Value:=EValue.AsDouble;
    except
     param.AsFloat:=0;
    end;
   end;
  tecurrency:
   begin
    try
     param.AsCurrency:=EValue.AsCurrency;
    except
     param.AsCurrency:=0;
    end;
   end;
 end;
}end;

procedure TFparamdef.FormCreate(Sender: TObject);
begin
 fparams:=TParams.create;
end;

procedure TFparamdef.FormDestroy(Sender: TObject);
begin
 fparams.free;
end;

procedure TFparamdef.ComboTypeClick(Sender: TObject);
var
 param:TParam;
begin
 param:=params.items[lparams.itemindex];
 // Is diferent?
 case ComboType.ItemIndex of
  0:
   param.DataType:=ftUnknown;
  1:
   param.DataType:=ftInteger;
  2:
   param.DataType:=ftFloat;
  3:
   param.DataType:=ftcurrency;
  7:
   param.DataType:=ftstring;
  4:
   param.DataType:=ftdate;
  5:
   param.DataType:=fttime;
  6:
   param.DataType:=ftdatetime;
  8:
   param.DataType:=ftblob;
 end;
 CheckNullClick(Self);
 LParamsClick(Lparams);
end;

procedure TFparamdef.CheckNullClick(Sender: TObject);
var
 param:TParam;
begin
 param:=params.Items[lparams.itemindex];
 // Redibuixem
 if checknull.checked then
 begin
  param.Value:=null;
 end
 else
 begin
  case ComboType.ItemIndex of
   0,7:
    begin
     param.DataType:=ftstring;
     param.Value:='Write a string';
    end;
   8:
    begin
     param.DataType:=ftBlob;
     param.SetBlobData(Pchar('Blob String'),11);
    end;
   1:
    begin
     param.DataType:=ftinteger;
     param.AsInteger:=0;
    end;
   2:
    begin
     param.DataType:=ftfloat;
     param.Asfloat:=0.0;
    end;
   3:
    begin
     param.DataType:=ftcurrency;
     param.AsCurrency:=0;
    end;
   4:
    begin
     param.DataType:=ftDate;
     param.AsDate:=date;
    end;
   5:
    begin
     param.DataType:=ftTime;
     param.AsTime:=time;
    end;
   6:
    begin
     param.DataType:=ftDateTime;
     param.AsDateTime:=now;
    end;
  end;
 end;
 lparamsclick(Self);
end;

procedure TFparamdef.BLoadClick(Sender: TObject);
begin
 if opendialog1.Execute then
 begin
  params.Items[lparams.itemindex].LoadFromFile(opendialog1.filename,ftBlob);
 end;
end;

procedure TFparamdef.DValueChange(Sender: TObject);
begin
{ if TValue.visible then
  params.items[lparams.itemindex].AsDatetime:=Trunc(DValue.DateTime)+Frac(TValue.datetime)
 else
  params.items[lparams.itemindex].AsDatetime:=Trunc(DValue.DateTime);
}end;

procedure TFparamdef.TValueChange(Sender: TObject);
begin
{ if DValue.visible then
  params.items[lparams.itemindex].AsDatetime:=Trunc(DValue.DateTime)+Frac(TValue.datetime)
 else
  params.items[lparams.itemindex].Astime:=TValue.Time;
}end;

end.
