{*******************************************************}
{                                                       }
{       IBAccess                                        }
{       Transaction options dialog                      }
{                                                       }
{       Author:                                         }
{       Toni Martir                                     }
{       http://www.ibaccess.org                         }
{       e-mail: toni@pala.com                           }
{                                                       }
{       OpenSource license (MPL)                        }
{                                                       }
{*******************************************************}

unit ibtranop;

interface

uses
  SysUtils, Classes, QControls, QForms, QDialogs,
  Db, ibdatabase, 
  ibacconst, DBClient, QStdCtrls, QExtCtrls, QGrids, QDBGrids;

type
  TFIBTranOp = class(TForm)
    GIsolation: TRadioGroup;
    L12a: TLabel;
    CheckWait: TCheckBox;
    L12b: TLabel;
    L1: TLabel;
    L2: TLabel;
    L34: TLabel;
    L3: TLabel;
    L4: TLabel;
    Checkreadonly: TCheckBox;
    GTables: TGroupBox;
    SComp: TDataSource;
    BOK: TButton;
    BCancel: TButton;
    GDatabases: TGroupBox;
    DTables: TClientdataset;
    CheckExclusive: TCheckBox;
    DTablesLock: TStringField;
    DTablesTable: TStringField;
    DBGrid1: TDBGrid;
    DLockType: TClientdataset;
    DLockTypeName: TStringField;
    DTablesLockname: TStringField;
    DDatabases: TClientdataset;
    SDatabases: TDataSource;
    DBGrid2: TDBGrid;
    DDatabasesIndex: TIntegerField;
    DDatabasenames: TClientdataset;
    DDatabasenamesIndex: TIntegerField;
    DDatabasenamesDatabase: TStringField;
    DDatabasesDatabase: TStringField;
    procedure GIsolationClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure DBGrid1EditButtonClick(Sender: TObject);
    procedure DDatabasesBeforeDelete(DataSet: TDataSet);
    procedure DDatabasesBeforePost(DataSet: TDataSet);
    procedure DDatabasesBeforeEdit(DataSet: TDataSet);
    procedure BOKClick(Sender: TObject);
  private
    { Private declarations }
   procedure Buildtrans;
  public
    { Public declarations }
    trans:TIBTransaction;
    dblist:TStrings;
  end;

procedure IBModifyTransaction(Trans:TIBTransaction;dblist:TStrings);

implementation

{$R *.xfm}

procedure IBModifyTransaction(Trans:TIBTransaction;dblist:TStrings);
var
 dia:TFIBTranOp;
begin
 dia:=TFIBTranop.create(Application);
 try
  dia.trans:=trans;
  dia.dblist:=dblist;
  dia.showmodal;
 finally
  dia.free;
 end;
end;


procedure TFIBTranOp.GIsolationClick(Sender: TObject);
begin
 L12A.Visible:=false;
 L12B.Visible:=false;
 L1.Visible:=false;
 L2.Visible:=false;
 L34.Visible:=false;
 L34.Visible:=false;
 L3.Visible:=false;
 L4.Visible:=false;
 case GIsolation.itemindex of
  0:
   begin
    L12A.Visible:=True;
    L12B.Visible:=True;
    L1.Visible:=true;
   end;
  1:
   begin
    L12A.Visible:=True;
    L12B.Visible:=True;
    L2.Visible:=true;
   end;
  2:
   begin
    L34.Visible:=True;
    L34.Visible:=True;
    L3.Visible:=true;
   end;
   else
   begin
    L34.Visible:=True;
    L34.Visible:=True;
    L4.Visible:=true;
   end;
 end;
end;

procedure TFIBTranOp.FormShow(Sender: TObject);
var
 index:integer;
 i:integer;
 table:string;
begin
 // Fills the transaction data
 GIsolation.itemindex:=2;
 index:=Trans.Params.IndexOf('read_committed');
 if index>=0 then
 begin
  index:=Trans.Params.IndexOf('rec_version');
  if index>=0 then
   GIsolation.itemindex:=0
  else
   GIsolation.itemindex:=1;
 end
 else
 begin
  index:=Trans.Params.IndexOf('consistency');
  if index>=0 then
   GIsolation.itemindex:=3;
 end;
 index:=Trans.Params.IndexOf('nowait');
 CheckWait.checked:=(index<0);
 index:=Trans.Params.IndexOf('read');
 CheckREadonly.Checked:=(index>=0);
 index:=Trans.Params.IndexOf('exclusive');
 CheckExclusive.Checked:=(index>=0);
 // Check for table reservation
 with trans.params do
 begin
  i:=0;
  While i<count do
  begin
   if ((Names[i]='lock_read') or (Names[i]='lock_write')) then
   begin
    if i+1>count then
     break;
    index:=Pos('=',Strings[i]);
    table:=Copy(Strings[i],index+1,50);
    if ((length(table)>0) and (strings[i+1]='protected')) then
    begin
     if Names[i]='lock_read' then
     begin
      DTables.Insertrecord(['read','read',table]);
     end
     else
     if Names[i]='lock_write' then
     begin
      DTables.Insertrecord(['write','write',table]);
     end;
    end;
    inc(i);
   end;
   inc(i);
  end;
 end;
 // Fill database list
 if Assigned(dblist) then
 begin
  for i:=0 to dblist.count-1 do
  begin
   DDatabasenames.Insertrecord([i,TIBDatabase(dblist.objects[i]).databasename]);
  end;
 end;
 for i:=0 to trans.DatabaseCount-1 do
 begin
  index:=dblist.IndexOfObject(trans.databases[i]);
  if index>=0 then
   DDatabases.Insertrecord([index]);
 end;
 ddatabases.beforepost:=ddatabasesbeforepost;
end;

procedure TFIBTranOp.FormCreate(Sender: TObject);
begin
 DLockType.CreateDataSet;
 DLockType.InsertRecord(['read']);
 DLockType.InsertRecord(['write']);
 Dtables.CreateDataSet;
 DDatabasenames.CreateDataSet;
 DDatabases.createdataset;
end;

procedure TFIBTranOp.DBGrid1EditButtonClick(Sender: TObject);
begin
 // Search for table names
 Trans.DefaultDatabase.GetTableNames(DBGrid1.Columns.Items[1].PickList);
 DBGrid1.Columns.Items[1].ButtonStyle:=cbsAuto;
 DBGrid1.Selectedfield:=DTablesLockName;
 DBGrid1.Selectedfield:=DTablesTable;
end;

procedure TFIBTranOp.DDatabasesBeforeDelete(DataSet: TDataSet);
var
 index:integer;
begin
 // It's de default?
 index:=DDatabasesindex.value;
 if assigned(dblist) then
 begin
  if dblist.objects[index]=trans.defaultdatabase then
   Raise Exception.Create(SrpWorkingdb);
 end;
end;

procedure TFIBTranOp.DDatabasesBeforePost(DataSet: TDataSet);
var
 index:integer;
begin
 index:=DDatabasesindex.value;
 if assigned(dblist) then
 begin
  if dblist.objects[index]=trans.defaultdatabase then
   Raise Exception.Create(SrpWorkingdbnoadd);
 end;
end;

procedure TFIBTranOp.DDatabasesBeforeEdit(DataSet: TDataSet);
var
 index:integer;
begin
 index:=DDatabasesindex.value;
 if assigned(dblist) then
 begin
  if dblist.objects[index]=trans.defaultdatabase then
   Abort;
 end;
end;

procedure TFIBTranOp.BOKClick(Sender: TObject);
begin
 Buildtrans;
 Close;
end;

procedure TFIBTranOp.Buildtrans;
var
 db:TIBDatabase;
begin
 DTables.checkbrowsemode;
 DDatabases.checkbrowsemode;
 Trans.Params.clear;
 if assigned(dblist) then
 begin
  db:=Trans.defaultdatabase;
  Trans.RemoveDatabases;
  DDatabases.first;
  While Not DDatabases.Eof do
  begin
   Trans.AddDatabase(TIBDatabase(dblist.objects[ddatabasesindex.value]));
   DDatabases.next;
  end;
  Trans.defaultdatabase:=db;
 end;
 if Checkreadonly.checked then
  Trans.params.add('read');
 if Not checkwait.checked then
  Trans.params.add('nowait');
 if CheckExclusive.checked then
  Trans.params.add('exclusive');
 // Build the TPB
 Case GIsolation.itemindex of
  0:
   begin
    Trans.params.add('read_committed');
    Trans.params.add('rec_version');
   end;
  1:
   begin
    Trans.params.add('read_committed');
   end;
  2:
   begin
    Trans.params.add('concurrency');
   end;
  else
  begin
   Trans.params.add('consistency');
  end;
 end;
 // Table reservation
 DTables.First;
 While Not DTables.Eof do
 begin
  if DTablesLock.AsString='read' then
  begin
   Trans.params.add('lock_read='+DTablesTable.asstring);
  end
  else
  begin
   Trans.params.add('lock_write='+DTablesTable.asstring);
  end;
  Trans.params.add('protected');
  DTables.next;
 end;
end;

end.
