{*******************************************************}
{                                                       }
{       IBAccess                                        }
{       Metadata to datatset utils                      }
{                                                       }
{       Author:                                         }
{       Toni Martir                                     }
{       http://www.ibaccess.org                         }
{       e-mail: toni@pala.com                           }
{                                                       }
{       OpenSource license (MPL)                        }
{                                                       }
{*******************************************************}

unit ibutil2;

interface


uses
  SysUtils, Classes,QForms,variants,
  ibcustomdataset,db,IBDatabaseInfo,IBDatabase,IBExternals,
  IBQuery, IBSql,ibmanst,ibacconst,ibacutils,IBServices,rpibdb;


type


  TIbdim=record
   low:integer;
   high:integer;
  end;
  TIBDefDomain=record
   name:string;
   basetype:integer;
   basetypestring:string;
   Notnull:boolean;
   arraysource:string;
   arraydim:array of Tibdim;
   arraydimcount:integer;
   check:string;
   default:string;
   computed:string;
   character_set:string;
   collation:string;
   size:integer;
   sizebytes:integer;
   scale:variant;
   precision:variant;
   precscale:variant;
   subtype:variant;
   segmentsize:variant;
   charsetid:integer;
   collationid:integer;
  end;

  TIBDefField=record
   name:string;
   domain:string;
   defdomain:TIBDefdomain;
   Notnull:boolean;
   default:string;
   computed:string;
   collation:string;
   order:integer;
  end;

  TIBDefconstraint=record
   name:string;
   keyname:string;
   checksource:string;
   fields:string;
   desttable:string;
   destfields:string;
   index:string;
   onupdate:string;
   ondelete:string;
  end;

  TIBDefIndex=record
   name:string;
   active:boolean;
   descending:boolean;
   unique:boolean;
   fieldcount:integer;
   fields:string;
   statistics:variant;
  end;

  TFIBUtil2 = class(TForm)
    IBDatabaseInfo1: TIBDatabaseInfo;
  private
    { Private declarations }
  public
    { Public declarations }
  end;



 function Filldatasetwithfieldsinfo(base:TIBDatabase;viewsystem:boolean;data:TDataset;table:string;fprogres:TFFetching):integer;
 function ibllegirmemo(dataset:TIBCustomDataset;camp:string):string;
 function FillDomaininfo(base:TIBDatabase;domain:string;fprogres:TFFetching):TIBDefdomain;
 function FillFieldinfo(base:TIBDatabase;table,field:string;fprogres:TFFetching):TIBDeffield;
 function SOurceFromFillDomainInfo(info:TIBDefdomain;dialect:integer):string;
 function ibfieldtypetosource(value:integer;size:integer;scale:integer;dialect:integer):string;
 function ibfieldtypetostringtype(value:integer;scale:integer;dialect:integer):string;
 function ibfieldtypetosourcescale(value:integer;size:integer;scale:integer;var precision,precscale:variant;dialect:integer):string;
 procedure IBExecuteSQL(base:TIBDatabase;sqltext:string);
 function IBObtainValueFromSQL(base:TIBDatabase;sqltext:string):variant;
 function DependentTypeToString(value:integer):string;
 function ConstraintTypeToString(value:string):string;
 function obtainyesno(value:boolean):string;
 procedure FillFieldsInfo(base:TIBDatabase;table:string;dataset:TDataset;fprogres:TFFetching);
 procedure FillDomainDepen(base:TIBDatabase;domain:string;dataset:TDataset;fprogres:TFFetching);
 procedure FillTableDepen(base:TIBDatabase;table:string;dataset:TDataset;fprogres:TFFetching);
 function GetTableOwner(base:TIBDatabase;table:string):string;
 procedure FillCheckInfos(base:TIBDatabase;table:string;dataset:TDataset;fprogres:TFFetching);
 procedure FillIndexInfos(base:TIBDatabase;table:string;dataset:TDataset;fprogres:TFFetching);
 procedure FillTriggerInfos(base:TIBDatabase;table:string;dataset:TDataset;fprogres:TFFetching);
 procedure Filldatasetwithdomainsinfo(base:TIBDatabase;viewsystem:boolean;systemdomains:boolean;data:TDataset;fprogres:TFFetching);
 function TriggerTypetostring(atype:integer):string;
 procedure FillWithUsers(user,password,server:string;protocol:TrpIBXProtocol;alist:TStrings);



implementation

{$R *.xfm}

function ibfieldtypetosource(value:integer;size:integer;scale:integer;dialect:integer):string;
begin
 case value of
  7:
   Result:='SMALLINT';
  8:
   Result:='INTEGER';
  9:
   Result:='QUAD';
  10:
   Result:='FLOAT';
  11:
   Result:='DOUBLE PRECISION';
  12:
   Result:='DATE';
  13:
   Result:='TIME';
  14:
   Result:='CHAR('+inttostr(size)+')';
  16:
   // INT64
   Result:='NUMERIC';
  27:
   Result:='DOUBLE PRECISION';
  35:
   if Dialect>2 then
    Result:='TIMESTAMP'
   else
    Result:='DATE';
  37:
   Result:='VARCHAR('+inttostr(size)+')';
  40:
   Result:='CSTRING('+inttostr(size)+')';
  261:
   begin
    Result:='BLOB';
   end;
  else
   Result:='';
 end;
 if scale<0 then
 begin
  Result:='NUMERIC';
  case size of
   2:
    REsult:=Result+'('+Inttostr(4)+','+inttostr(-scale)+')';
   4:
    REsult:=Result+'('+Inttostr(9)+','+inttostr(-scale)+')';
   8:
    begin
     if dialect=1 then
      REsult:=Result+'('+Inttostr(15)+','+inttostr(-scale)+')'
     else
      REsult:=Result+'('+Inttostr(18)+','+inttostr(-scale)+')'
    end;
   else
     REsult:=Result+'('+Inttostr(15)+','+inttostr(-scale)+')';
  end;
 end
 else
 begin
  if Result='NUMERIC' then
   Result:='NUMERIC(18,0)';
 end;
end;

function ibfieldtypetostringtype(value:integer;scale:integer;dialect:integer):string;
begin
 case value of
  7:
   Result:='SMALLINT';
  8:
   Result:='INTEGER';
  9:
   Result:='QUAD';
  10:
   Result:='FLOAT';
  11:
   Result:='DOUBLE PRECISION';
  12:
   Result:='DATE';
  13:
   Result:='TIME';
  14:
   Result:='CHAR';
  16:
   // INT64
   Result:='NUMERIC';
  27:
   Result:='DOUBLE PRECISION';
  35:
   if Dialect>2 then
    Result:='TIMESTAMP'
   else
    Result:='DATE';
  37:
   Result:='VARCHAR';
  40:
   Result:='CSTRING';
  261:
   begin
    Result:='BLOB';
   end;
  else
   Result:='';
 end;
 if scale<0 then
  result:='NUMERIC';
end;

function ibfieldtypetosourcescale(value:integer;size:integer;scale:integer;var precision,precscale:variant;dialect:integer):string;
begin
 precision:=null;
 precscale:=null;
 case value of
  7:
   Result:='SMALLINT';
  8:
   Result:='INTEGER';
  9:
   Result:='QUAD';
  10:
   Result:='FLOAT';
  11:
   Result:='DOUBLE PRECISION';
  12:
   Result:='DATE';
  13:
   Result:='TIME';
  14:
   begin
    Result:='CHAR';
   end;
  16:
   // INT64
   Result:='NUMERIC';
  27:
   Result:='DOUBLE PRECISION';
  35:
   if Dialect>2 then
    Result:='TIMESTAMP'
   else
    Result:='DATE';
  37:
   begin
    Result:='VARCHAR';
   end;
  40:
   begin
    Result:='CSTRING';
   end;
  261:
   begin
    Result:='BLOB';
   end;
  else
   Result:='';
 end;
 if scale<0 then
 begin
  Result:='NUMERIC';
  case size of
   2:
    begin
     precision:=4;
     precscale:=-scale;
    end;
   4:
    begin
     precision:=9;
     precscale:=-scale;
    end;
   8:
    begin
     if dialect=1 then
      precision:=15
     else
      precision:=18;
     precscale:=-scale;
    end;
   else
    begin
     precision:=15;
     precscale:=-scale;
    end;
  end;
 end
 else
 begin
  if Result='NUMERIC' then
   Result:='NUMERIC(18,0)';
 end;
end;


function ibllegirmemo(dataset:TIBCustomDataset;camp:string):string;
var
 stream:TStream;
begin
 stream:=dataset.CreateBlobStream(dataset.fieldbyname(camp),bmread);
 try
  SetLength(Result,stream.Size);
  stream.Read(Pchar(result)^,stream.size);
 finally
  stream.free;
 end;
end;




function FillDomainfromquerys(QDomini:TIBSQL;QBusca:TIBSQL;dialect:integer;fprogres:TFFetching):TIBDefdomain;
var
 primera:boolean;
 i:integer;
 collid:variant;
begin
      Result.collation:='';
      Result.name:=QDomini.FieldByName('DOMINI').AsString;
      Result.arraydimcount:=0;
      // Busquem el tipus de camp
      Result.basetype:=QDomini.FieldByName('TIPO').AsInteger;
      // Mirem si pot ser null
      if Not QDomini.FieldByName('NONULO').isnull then
      begin
       Result.notnull:=(QDomini.FieldByName('NONULO').AsInteger=1);
      end
      else
       Result.notnull:=False;
      if (QDomini.FieldByName('DIMENSIO').AsInteger=0) then
       Result.arraysource:=''
      else
      begin
       // Search for dimensions
       QBusca.sql.text:='SELECT COUNT(RDB$DIMENSION) DIMCOUNT '+
       'FROM RDB$FIELD_DIMENSIONS WHERE RDB$FIELD_NAME='''+QDomini.FieldByName('DOMINI').AsString+'''';
       QBusca.ExecQuery;
       try
        Result.arraydimcount:=QBusca.fieldbyname('DIMCOUNT').AsInteger;
        SetLength(REsult.arraydim,Result.arraydimcount);
       finally
        QBusca.close;
       end;
       i:=0;
       if Result.arraydimcount>0 then
       begin
        QBusca.sql.text:='SELECT RDB$DIMENSION DIM,RDB$LOWER_BOUND LOWDIM,'+
         'RDB$UPPER_BOUND HIGHDIM '+
        'FROM RDB$FIELD_DIMENSIONS WHERE RDB$FIELD_NAME='''+QDomini.FieldByName('DOMINI').AsString+''''+
        ' ORDER BY 1';
        QBusca.ExecQuery;
        try
         if Not (QBusca.EOF AND QBusca.BOF) then
         begin
          primera:=true;
          Result.arraysource:='[';
          While Not QBusca.eof do
          begin
           if not primera then
           begin
            Result.arraysource:=Result.arraysource+',';
           end
           else
            primera:=false;
           Result.arraysource:=Result.arraysource+QBusca.Fieldbyname('LOWDIM').AsString+
           ':'+QBusca.Fieldbyname('HIGHDIM').AsString;
           Result.arraydim[i].Low:=QBusca.Fieldbyname('LOWDIM').Asinteger;
           Result.arraydim[i].High:=QBusca.Fieldbyname('HIGHDIM').Asinteger;
           inc(i);
           QBusca.Next;
           if assigned(fprogres)then
           begin
            Application.processmessages;
            if fprogres.cancel then
             Raise Exception.Create(SRpOperationAborted);
           end;
          end;
          Result.arraysource:=Result.arraysource+']';
         end
         else
         begin
          Result.arraysource:='';
         end;
        finally
         QBusca.Close;
        end;
       end;
      end;
      REsult.check:=Trim(QDomini.FieldByName('VALIDA').Asstring);
      Result.default:=Trim(QDomini.FieldByname('OMISION').Asstring);
      Result.computed:=Trim(QDomini.FieldByname('CALCULA').Asstring);
      Result.segmentsize:=QDomini.FieldByname('SEGLEN').AsVariant;
      Result.subtype:=QDomini.FieldByname('SUBTIPO').AsVariant;
      if QDomini.FieldByName('TAMANY').Isnull then
      begin
       Result.size:=0;
      end
      else
      begin
       Result.size:=QDomini.FieldByName('TAMANY').AsInteger;
      end;
      Result.sizebytes:=Result.size;
      Result.collationid:=0;
      if Not QDomini.FieldByName('CHARLEN').isnull then
      begin
       if Not (QDomini.FieldByName('CHARLEN').AsInteger=0) then
        Result.size:=QDomini.FieldByName('CHARLEN').AsInteger;
      end;
      Result.scale:=QDomini.FieldByName('ESCALA').AsVariant;
      if QDomini.FieldByName('CHARSET').Isnull then
      begin
       Result.character_set:='';
       Result.charsetid:=0;
      end
      else
      begin
       Result.charsetid:=QDomini.FieldByName('CHARSET').AsInteger;
       // Busquem el character set
       QBusca.sql.text:='SELECT RDB$CHARACTER_SET_NAME NOM '+
       'FROM RDB$CHARACTER_SETS WHERE RDB$CHARACTER_SET_ID='+
       Trim(QDomini.FieldByName('CHARSET').AsString);
       QBusca.ExecQuery;
       try
        if Not (QBusca.EOF AND QBusca.BOF) then
        begin
         Result.character_set:=Trim(QBusca.FieldByname('NOM').AsString);
         i:=QDomini.FieldIndex['ORDRE1'];
         if i<0 then
          collid:=QDomini.FieldByname('ORDRE').AsVariant
         else
         begin
          collid:=QDomini.FieldByname('ORDRE1').AsVariant;
          if collid=null then
           collid:=QDomini.FieldByname('ORDRE').AsVariant;
         end;
         if Not (collid=null) then
         begin
          QBusca.Close;
          QBusca.SQL.TExt:='SELECT RDB$COLLATION_NAME NOM '+
          'FROM RDB$COLLATIONS WHERE RDB$CHARACTER_SET_ID='+
          QDomini.FieldByName('CHARSET').AsString+
          ' AND RDB$COLLATION_ID='+IntToStr(collid);
          QBusca.ExecQuery;
          if (QBusca.EOF AND QBusca.BOF) then
          begin
           Result.collation:='';
           Result.collationid:=0;
          end
          else
          begin
           Result.Collation:=QBusca.fieldbyname('NOM').Value;
           Result.collationid:=collid;
          end;
         end
         else
         begin
          Result.collation:='';
         end;
        end
        else
        begin
         Result.character_set:='';
        end;
       finally
        QBusca.Close;
       end;
      end;
      Result.precision:=null;
      Result.precscale:=null;
      Result.basetypestring:=ibfieldtypetosourcescale(Result.basetype,Result.size,Result.scale,Result.precision,Result.precscale,dialect);
end;


function FillDomaininfo(base:TIBDatabase;domain:string;fprogres:TFFetching):TIBDefdomain;
var
 Tran1:TIBTransaction;
 QDomini:TIBSQL;
 QBusca:TIBSQL;
begin
 Result.name:=domain;
 Tran1:=TIBTransaction.create(Application);
 try
  tran1.DefaultDatabase:=base;
  QDomini:=TIBSQL.Create(Application);
  QBusca:=TIBSQL.Create(Application);
  try
   QBusca.database:=base;
   QBusca.transaction:=tran1;
   QDomini.database:=base;
   QDomini.transaction:=tran1;
   tran1.StartTransaction;
   try
    QDomini.SQL.TExt:='SELECT RDB$FIELD_NAME DOMINI,RDB$VALIDATION_SOURCE VALIDA,RDB$COMPUTED_SOURCE CALCULA,'+
     'RDB$DEFAULT_SOURCE OMISION,RDB$FIELD_LENGTH TAMANY,RDB$SEGMENT_LENGTH SEGLEN,'+
     'RDB$FIELD_SCALE ESCALA,RDB$NULL_FLAG NONULO,'+
     'RDB$CHARACTER_SET_ID CHARSET,RDB$COLLATION_ID ORDRE,'+
      'RDB$FIELD_TYPE TIPO,RDB$FIELD_SUB_TYPE SUBTIPO,'+
     'RDB$DIMENSIONS DIMENSIO,RDB$CHARACTER_LENGTH CHARLEN'+
     ' FROM RDB$FIELDS WHERE RDB$FIELD_NAME='''+domain+'''';
    QDomini.ExecQuery;
    try
     if assigned(fprogres)then
     begin
      Application.processmessages;
      if fprogres.cancel then
       Raise Exception.Create(SRpOperationAborted);
     end;
     if Not (QDomini.BOF AND QDomini.EOF) then
     begin
      REsult:=FillDomainFromQuerys(QDomini,QBusca,base.sqldialect,fprogres);
     end
     else
      Raise Exception.create(SRpDomainnotfound);
    finally
     QDomini.CLose;
    end;
    tran1.commitretaining;
   except
    tran1.rollback;
    raise;
   end;
  finally
   QBusca.free;
   QDomini.free;
  end;
 finally
  tran1.free;
 end;
end;

function SOurceFromFillDomainInfo(info:TIBDefdomain;dialect:integer):string;
var
 cadena:string;
begin
 cadena:='CREATE DOMAIN '+info.name+' AS ';
 cadena:=cadena+ibfieldtypetosource(info.basetype,info.size,info.scale,dialect)+#13+#10;
 if (info.basetype=261) then
 begin
  if (Not info.subtype=null) then
  begin
   cadena:=cadena+' SUB_TYPE '+inttostr(info.subtype);
  end;
  if (Not info.segmentsize=null) then
  begin
   cadena:=cadena+' SEGMENT_SIZE '+inttostr(info.segmentsize);
  end;
 end;
 cadena:=cadena+info.arraysource+#13+#10;

 if info.Notnull then
  cadena:=cadena+'NOT NULL'+#13+#10;

 if length(info.character_set)>0 then
  cadena:=cadena+'CHARACTER SET '+info.character_set+#13+#10;

 if Length(info.default)>0 then
  cadena:=cadena+info.default+#13+#10;
 if Length(info.check)>0 then
  cadena:=cadena+info.check+#13+#10;
 if length(info.collation)>0 then
  cadena:=cadena+'COLLATE '+info.collation+#13+#10;
 Result:=cadena;
end;

procedure IBExecuteSQL(base:TIBDatabase;sqltext:string);
var
 IBCustom:TIBSQL;
 IBtran:TIBTransaction;
begin
 IBCustom:=TIBSQL.Create(Application);
 IBtran:=TIBTransaction.create(Application);
 try
  IBCustom.paramcheck:=false;
  IBTran.defaultdatabase:=base;
  IBCustom.database:=base;
  IBCUstom.transaction:=ibtran;
  IBtran.StartTransaction;
  try
   IBCustom.SQL.text:=sqltext;
   IBCustom.execquery;
   IBTran.commit;
  except
   IBTran.rollback;
   raise;
  end;
 finally
  IBCustom.free;
  IBTran.free;
 end;
end;

function IBObtainValueFromSQL(base:TIBDatabase;sqltext:string):variant;
var
 IBCustom:TIBSQL;
 IBtran:TIBTransaction;
begin
 Result:=null;
 IBCustom:=TIBSQL.Create(Application);
 IBtran:=TIBTransaction.create(Application);
 try
  IBCustom.ParamCheck:=false;
  IBTran.defaultdatabase:=base;
  IBCustom.database:=base;
  IBCUstom.transaction:=ibtran;
  IBtran.StartTransaction;
  try
   IBCustom.SQL.text:=sqltext;
   IBCustom.execquery;
   if Not IBCustom.EOF then
   begin
    Result:=IBCustom.fields[0].Value;
   end;
   IBTran.commit;
  except
   IBTran.rollback;
   raise;
  end;
 finally
  IBCustom.free;
  IBTran.free;
 end;
end;

function DependentTypeToString(value:integer):string;
begin
 Result:='';
 case value of
  0:
   REsult:=SRpTable;
  1:
   Result:=SRpView;
  2:
   Result:=SRpTrigger;
  3:
   Result:=SRpComputedfield;
  4:
   Result:=SRpValidation;
  5:
   Result:=SRpProcedure;
  6:
   Result:=SRpExpresionIndex;
  7:
   Result:=SRpException;
  8:
   Result:=SRpUser;
  9:
   Result:=SRpField;
  10:
   Result:=SRpIndex;
 end;
end;

function ConstraintTypeToString(value:string):string;
begin
 value:=trim(value);
 Result:='';
 if value='PRIMARY KEY' then
 begin
  Result:=SIBPrimarykey;
  exit;
 end;
 if value='UNIQUE' then
 begin
  Result:=SIBUnique;
  exit;
 end;
 if value='NOT NULL' then
 begin
  Result:=SIBNotNull;
  exit;
 end;
 if value='CHECK' then
 begin
  Result:=SIBCheck;
  exit;
 end;
 if value='FOREIGN KEY' then
 begin
  Result:=SIBForeignkey;
  exit;
 end;
end;


function obtainyesno(value:boolean):string;
begin
 if value then
  Result:=SIBYes
 else
  Result:=SIBNo;
end;

procedure FillFieldsInfo(base:TIBDatabase;table:string;dataset:TDataset;fprogres:TFFetching);
var
 result:TIBDeffield;
 Tran1:TIBTransaction;
 QTemp:TIBSQL;
 QBusca:TIBSQL;
begin
 dataset.disablecontrols;
 try
  Tran1:=TIBTransaction.create(Application);
  try
   tran1.DefaultDatabase:=base;
   QTemp:=TIBSQL.Create(Application);
   QBusca:=TIBSQL.Create(Application);
   try
    QBusca.database:=base;
    QBusca.transaction:=tran1;
    QTemp.database:=base;
    QTemp.transaction:=tran1;
    tran1.StartTransaction;
    try
     QTemp.SQL.text:='SELECT F.RDB$FIELD_NAME CAMP,D.RDB$FIELD_NAME DOMINI,'+
     ' F.RDB$FIELD_SOURCE SOURCE,F.RDB$FIELD_POSITION POSICIO,'+
     ' F.RDB$DEFAULT_SOURCE DEFVALUE,F.RDB$NULL_FLAG NULO,'+
     ' F.RDB$SECURITY_CLASS,F.RDB$COLLATION_ID ORDREC,'+
     ' D.RDB$VALIDATION_SOURCE VALIDA,D.RDB$COMPUTED_SOURCE CALCULA,'+
     ' D.RDB$DEFAULT_SOURCE OMISION,D.RDB$FIELD_LENGTH TAMANY,D.RDB$SEGMENT_LENGTH SEGLEN,'+
     ' D.RDB$FIELD_SCALE ESCALA,D.RDB$NULL_FLAG NONULO,'+
     ' D.RDB$CHARACTER_LENGTH CHARLEN,D.RDB$DIMENSIONS DIMENSIO,'+
     ' D.RDB$CHARACTER_SET_ID CHARSET,D.RDB$COLLATION_ID ORDRE,'+
     ' D.RDB$FIELD_TYPE TIPO,D.RDB$FIELD_SUB_TYPE SUBTIPO'+
     ' FROM RDB$RELATION_FIELDS F'+
     ' LEFT OUTER JOIN RDB$FIELDS D ON D.RDB$FIELD_NAME=F.RDB$FIELD_SOURCE'+
     ' WHERE F.RDB$RELATION_NAME='''+table+''''+
     ' ORDER BY RDB$FIELD_POSITION';
     QTemp.ExecQuery;
     try
      While Not QTemp.EOF do
      begin
       Result.defdomain:=FillDomainfromquerys(QTemp,QBusca,base.sqldialect,fprogres);
       Result.collation:=Result.defdomain.collation;
       // Primer emplenem el domini
       if pos('RDB$',Result.defdomain.name)=1 then
        Result.defdomain.name:='';
       if QTemp.fieldbyname('DEFVALUE').Isnull then
       begin
        Result.default:=REsult.defdomain.default;
       end
       else
        if Length(QTemp.fieldbyname('DEFVALUE').AsString)>0 then
        begin
         Result.default:=QTemp.fieldbyname('DEFVALUE').AsString;
        end
        else
         Result.default:=result.defdomain.default;
       if QTemp.fieldbyname('NULO').isnull then
        Result.notnull:=REsult.defdomain.notnull
       else
        Result.notnull:=(QTemp.fieldbyname('NULO').ASInteger=1);

        try
          if Not QTemp.FieldByName('ORDREC').isnull then
          begin
           if Not QTemp.FieldByName('ORDREC').AsInteger<>result.defdomain.collationid then
           begin
            QBusca.Close;
            QBusca.SQL.TExt:='SELECT RDB$COLLATION_NAME NOM '+
            'FROM RDB$COLLATIONS WHERE RDB$CHARACTER_SET_ID='+
            IntToStr(Result.defdomain.charsetid)+' AND RDB$COLLATION_ID='+
            QTemp.FieldByName('ORDREC').AsString;
            QBusca.ExecQuery;
            if (QBusca.EOF AND QBusca.BOF) then
            begin
             Result.collation:='';
            end
            else
            begin
             Result.Collation:=QBusca.fieldbyname('NOM').Value;
            end;
           end
           else
           begin
            Result.collation:='';
           end;
          end;
        finally
         QBusca.Close;
        end;
        Result.order:=QTemp.fieldbyname('POSICIO').Asinteger;
        if Pos('RDB$',Result.defdomain.name)=1 then
         Result.defdomain.name:='';
        // Assignem
        dataset.append;
        try
         dataset.Fieldbyname('nom').asstring:=QTemp.fieldbyname('CAMP').AsString;
         dataset.Fieldbyname('dominio').asstring:=Result.defdomain.name;
         dataset.Fieldbyname('default').asstring:=Result.default;
         dataset.Fieldbyname('nuls').asstring:=Obtainyesno(not Result.notnull);
         dataset.Fieldbyname('check').asstring:=Result.defdomain.check;
         dataset.Fieldbyname('size').asinteger:=Result.defdomain.size;
         dataset.Fieldbyname('precision').AsVariant:=Result.defdomain.precision;
         dataset.Fieldbyname('scale').AsVariant:=Result.defdomain.precscale;
         dataset.Fieldbyname('subtype').asvariant:=Result.defdomain.subtype;
         dataset.Fieldbyname('segmentsize').asvariant:=Result.defdomain.segmentsize;
         dataset.Fieldbyname('collation').Asstring:=Result.collation;
         dataset.Fieldbyname('charset').asstring:=Result.defdomain.character_set;
         dataset.Fieldbyname('Array').asstring:=Result.defdomain.arraysource;
         dataset.Fieldbyname('computed').asstring:=Result.defdomain.computed;
         dataset.Fieldbyname('basetype').asstring:=Result.defdomain.basetypestring;
         dataset.Fieldbyname('Orden').asinteger:=REsult.order;
         dataset.post;
        except
         dataset.cancel;
         raise;
        end;
       QTemp.next;
       if assigned(fprogres)then
       begin
        Application.processmessages;
        if fprogres.cancel then
         Raise Exception.Create(SRpOperationAborted);
       end;
      end;
     finally
      QTemp.CLose;
     end;
     tran1.commitretaining;
     dataset.first;
    except
     tran1.rollback;
     raise;
    end;
   finally
    QBusca.free;
    QTemp.free;
   end;
  finally
   tran1.free;
  end;
 finally
  dataset.enablecontrols;
 end;
end;

procedure FillDomainDepen(base:TIBDatabase;domain:string;dataset:TDataset;fprogres:TFFetching);
var
 Tran1:TIBTransaction;
 QTemp:TIBSQL;
begin
 dataset.disablecontrols;
 try
  Tran1:=TIBTransaction.create(Application);
  try
   tran1.DefaultDatabase:=base;
   QTemp:=TIBSQL.Create(Application);
   try
    QTemp.database:=base;
    QTemp.transaction:=tran1;
    tran1.StartTransaction;
    try
     QTemp.SQL.TExt:='SELECT RDB$RELATION_NAME ATABLE,RDB$FIELD_NAME AFIELD'+
      ' FROM RDB$RELATION_FIELDS WHERE RDB$FIELD_SOURCE='''+domain+
      ''' ORDER BY 1,2';
     QTemp.ExecQuery;
     try
      While Not QTemp.EOF do
      begin
        // Assignem
       dataset.append;
       try
        dataset.fieldbyname('TABLE').AsString:=Trim(QTemp.FieldByname('ATABLE').Asstring);
        dataset.fieldbyname('FIELD').AsString:=Trim(QTemp.FieldByname('AFIELD').Asstring);
        dataset.post;
       except
        dataset.cancel;
        raise;
       end;
       QTemp.next;
       if assigned(fprogres)then
       begin
        Application.processmessages;
        if fprogres.cancel then
         Raise Exception.Create(SRpOperationAborted);
       end;
      end;
     finally
      QTemp.CLose;
     end;
     tran1.commitretaining;
     dataset.first;
    except
     tran1.rollback;
     raise;
    end;
   finally
    QTemp.free;
   end;
  finally
   tran1.free;
  end;
 finally
  dataset.enablecontrols;
 end;
end;

function GetTableOwner(base:TIBDatabase;table:string):string;
var
 Tran1:TIBTransaction;
 QTemp:TIBSQL;
begin
 Result:='';
 Tran1:=TIBTransaction.create(Application);
 try
  tran1.DefaultDatabase:=base;
  QTemp:=TIBSQL.Create(Application);
  try
   QTemp.database:=base;
   QTemp.transaction:=tran1;
   tran1.StartTransaction;
   try
    QTemp.SQL.text:='SELECT RDB$OWNER_NAME OWNAME,'+
    'RDB$RELATION_NAME NOM FROM RDB$RELATIONS '+
    'WHERE RDB$RELATION_NAME='''+table+'''';
    QTemp.ExecQuery;
    try
     if Not QTemp.EOF then
     begin
      Result:=QTemp.fieldbyname('OWNAME').AsString;
     end;
    finally
     QTemp.CLose;
    end;
    tran1.commitretaining;
   except
    tran1.rollback;
    raise;
   end;
  finally
   QTemp.free;
  end;
 finally
  tran1.free;
 end;
end;

procedure FillTableDepen(base:TIBDatabase;table:string;dataset:TDataset;fprogres:TFFetching);
var
 Tran1:TIBTransaction;
 QTemp:TIBSQL;
begin
 dataset.disablecontrols;
 try
  Tran1:=TIBTransaction.create(Application);
  try
   tran1.DefaultDatabase:=base;
   QTemp:=TIBSQL.Create(Application);
   try
    QTemp.database:=base;
    QTemp.transaction:=tran1;
    tran1.StartTransaction;
    try
     QTemp.SQL.text:='SELECT RDB$DEPENDENT_NAME DEPNAME,'+
     ' RDB$FIELD_NAME CAMP,RDB$DEPENDENT_TYPE TYP1,RDB$DEPENDED_ON_TYPE TYP2 '+
     'FROM RDB$DEPENDENCIES WHERE RDB$DEPENDED_ON_NAME='''+table+''''+
     ' ORDER BY 1,2';
     QTemp.ExecQuery;
     try
      While Not QTemp.EOF do
      begin
        // Assignem
       dataset.append;
       try
        dataset.fields[0].AsString:=QTemp.fieldbyname('DEPNAME').AsString;
        dataset.fields[1].AsString:=DependentTypeToString(QTemp.fieldbyname('TYP1').AsInteger);
        dataset.fields[2].AsString:=QTemp.fieldbyname('CAMP').AsString;
        dataset.post;
       except
        dataset.cancel;
        raise;
       end;
       QTemp.next;
       if assigned(fprogres)then
       begin
        Application.processmessages;
        if fprogres.cancel then
         Raise Exception.Create(SRpOperationAborted);
       end;
      end;
     finally
      QTemp.CLose;
     end;
     tran1.commitretaining;
     dataset.first;
    except
     tran1.rollback;
     raise;
    end;
   finally
    QTemp.free;
   end;
  finally
   tran1.free;
  end;
 finally
  dataset.enablecontrols;
 end;
end;

function Fillconstraintfromquery(QTemp:TIBSQL;QBusca:TIBSQL;fprogres:TFFetching):TIBDefconstraint;
var
 campo,campos,integ2,index2:string;
 descripcion,actualiza,borra,tabla:string;
 trigger:string;
begin
 campo:='';
 campos:='';
 actualiza:='';
 borra:='';
 descripcion:='';
 tabla:='';
 // Constraint type?
 if Trim(QTemp.fieldbyname('TIPO').Asstring)='NOT NULL' then
 begin
  QBusca.close;
  QBusca.sql.text:='SELECT RDB$TRIGGER_NAME CAMPO'+
   ' FROM RDB$CHECK_CONSTRAINTS '+
   ' WHERE RDB$CONSTRAINT_NAME='''+QTemp.fieldbyname('CONNAME').AsString+'''';
  QBusca.execquery;
  if not QBusca.eof then
   campo:=QBusca.FieldByName('CAMPO').Asstring;
 end
 else
 if Trim(QTemp.fieldbyname('TIPO').Asstring)='PRIMARY KEY' then
 begin
  QBusca.close;
  QBusca.sql.text:='SELECT RDB$FIELD_NAME CAMPO,RDB$FIELD_POSITION POSICIO'+
   ' FROM RDB$INDEX_SEGMENTS '+
   ' WHERE RDB$INDEX_NAME='''+QTemp.fieldbyname('INDICE').AsString+''''+
   ' ORDER BY 2';
  QBusca.execquery;
  While not QBusca.eof do
  begin
   if campo='' then
    campo:=Trim(QBusca.FieldByName('CAMPO').Asstring)
   else
    campo:=campo+','+Trim(QBusca.FieldByName('CAMPO').Asstring);
   QBusca.next;
   if assigned(fprogres)then
   begin
    Application.processmessages;
    if fprogres.cancel then
     Raise Exception.Create(SRpOperationAborted);
   end;
  end;
 end
 else
 if Trim(QTemp.fieldbyname('TIPO').Asstring)='UNIQUE' then
 begin
  QBusca.close;
  QBusca.sql.text:='SELECT RDB$FIELD_NAME CAMPO,RDB$FIELD_POSITION POSICIO'+
   ' FROM RDB$INDEX_SEGMENTS '+
   ' WHERE RDB$INDEX_NAME='''+QTemp.fieldbyname('INDICE').AsString+''''+
   ' ORDER BY 2';
  QBusca.execquery;
  While not QBusca.eof do
  begin
   if campo='' then
    campo:=Trim(QBusca.FieldByName('CAMPO').Asstring)
   else
    campo:=campo+','+Trim(QBusca.FieldByName('CAMPO').Asstring);
   QBusca.next;
   if assigned(fprogres)then
   begin
    Application.processmessages;
    if fprogres.cancel then
     Raise Exception.Create(SRpOperationAborted);
   end;
  end;
 end
 else
 if Trim(QTemp.fieldbyname('TIPO').Asstring)='FOREIGN KEY' then
 begin
  // Fields
  QBusca.close;
  QBusca.sql.text:='SELECT RDB$FIELD_NAME CAMPO,RDB$FIELD_POSITION POSICIO'+
   ' FROM RDB$INDEX_SEGMENTS '+
   ' WHERE RDB$INDEX_NAME='''+QTemp.fieldbyname('INDICE').AsString+''''+
   ' ORDER BY 2';
  QBusca.execquery;
  While not QBusca.eof do
  begin
   if campo='' then
    campo:=Trim(QBusca.FieldByName('CAMPO').Asstring)
   else
    campo:=campo+','+Trim(QBusca.FieldByName('CAMPO').Asstring);
   QBusca.next;
   if assigned(fprogres)then
   begin
    Application.processmessages;
    if fprogres.cancel then
     Raise Exception.Create(SRpOperationAborted);
   end;
  end;
  // Constraint type
  QBusca.close;
  QBusca.sql.text:='SELECT RDB$CONST_NAME_UQ CONNAME,RDB$UPDATE_RULE ACTUALIZA, '+
   ' RDB$DELETE_RULE BORRA'+
   ' FROM RDB$REF_CONSTRAINTS '+
   ' WHERE RDB$CONSTRAINT_NAME='''+QTemp.fieldbyname('CONNAME').AsString+'''';
  QBusca.execquery;
  if not QBusca.eof then
  begin
   integ2:=QBusca.fieldbyname('CONNAME').asstring;
   actualiza:=QBusca.fieldbyname('ACTUALIZA').asstring;
   borra:=QBusca.fieldbyname('BORRA').asstring;
   // Table referenced
   QBusca.close;
   QBusca.SQL.text:='SELECT RDB$INDEX_NAME INDICE,RDB$RELATION_NAME TABLA '+
    'FROM RDB$RELATION_CONSTRAINTS WHERE RDB$CONSTRAINT_NAME='''+integ2+'''';
   QBusca.execquery;
   if not QBusca.eof then
   begin
    index2:=QBusca.FieldByName('INDICE').Asstring;
    Tabla:=QBusca.FieldByName('TABLA').Asstring;
    // Final fields
    QBusca.close;
    QBusca.sql.text:='SELECT RDB$FIELD_NAME CAMPO,RDB$FIELD_POSITION POSICIO'+
     ' FROM RDB$INDEX_SEGMENTS '+
     ' WHERE RDB$INDEX_NAME='''+index2+''''+
     ' ORDER BY 2';
    QBusca.execquery;
    While not QBusca.eof do
    begin
     if campos='' then
      campos:=Trim(QBusca.FieldByName('CAMPO').Asstring)
     else
      campos:=campos+','+Trim(QBusca.FieldByName('CAMPO').Asstring);
     QBusca.next;
     if assigned(fprogres)then
     begin
      Application.processmessages;
      if fprogres.cancel then
      Raise Exception.Create(SRpOperationAborted);
     end;
    end;
   end;
  end;
 end
 else
 if Trim(QTemp.fieldbyname('TIPO').Asstring)='CHECK' then
 begin
  // Search the check constraint
  QBusca.close;
  QBusca.sql.text:='SELECT RDB$TRIGGER_NAME CAMPO'+
   ' FROM RDB$CHECK_CONSTRAINTS '+
   ' WHERE RDB$CONSTRAINT_NAME='''+QTemp.fieldbyname('CONNAME').AsString+'''';
  QBusca.execquery;
  if not QBusca.eof then
  begin
   trigger:=QBusca.FieldByName('CAMPO').Asstring;
   QBusca.close;
   QBusca.sql.text:='SELECT RDB$TRIGGER_SOURCE '+
    ' FROM RDB$TRIGGERS '+
    ' WHERE RDB$TRIGGER_NAME='''+trigger+'''';
   QBusca.execquery;
   if Not QBusca.eof then
   begin
    descripcion:=Qbusca.fieldbyname('RDB$TRIGGER_SOURCE').AsString;
   end;
  end;
 end;
 Result.name:=QTemp.fieldbyname('CONNAME').AsString;
 Result.keyname:=ConstraintTypeToString(QTemp.fieldbyname('TIPO').Asstring);
 Result.fields:=campo;
 Result.onupdate:=actualiza;
 Result.ondelete:=borra;
 Result.destfields:=campos;
 Result.desttable:=tabla;
 Result.checksource:=descripcion;
 REsult.index:=QTemp.fieldbyname('INDICE').AsString;
end;




function Fillindexfromquery(QTemp:TIBSQL;QBusca:TIBSQL;fprogres:TFFetching):TIBDefIndex;
var
 campo:string;
begin
 QBusca.close;
 campo:='';
 QBusca.sql.text:='SELECT RDB$FIELD_NAME CAMPO,RDB$FIELD_POSITION POSICIO'+
  ' FROM RDB$INDEX_SEGMENTS '+
  ' WHERE RDB$INDEX_NAME='''+QTemp.fieldbyname('NOMBRE').AsString+''''+
  ' ORDER BY 2';
 QBusca.execquery;
 While not QBusca.eof do
 begin
  if campo='' then
   campo:=Trim(QBusca.FieldByName('CAMPO').Asstring)
  else
   campo:=campo+','+Trim(QBusca.FieldByName('CAMPO').Asstring);
  QBusca.next;
  if assigned(fprogres)then
  begin
   Application.processmessages;
   if fprogres.cancel then
    Raise Exception.Create(SRpOperationAborted);
  end;
 end;
 Result.name:=QTemp.fieldbyname('NOMBRE').AsString;
 Result.fields:=campo;
 REsult.active:=(QTemp.fieldbyname('INACTIVO').Asinteger=0);
 REsult.unique:=(QTemp.fieldbyname('UNICO').Asinteger=1);
 Result.fieldcount:=QTemp.fieldbyname('FIELDCOUNT').Asinteger;
 REsult.descending:=(QTemp.fieldbyname('TIPO').Asinteger=2);
 Result.statistics:=QTemp.fieldbyname('ESTADIST').AsVariant;
end;


procedure FillIndexInfos(base:TIBDatabase;table:string;dataset:TDataset;fprogres:TFFetching);
var
 Tran1:TIBTransaction;
 QTemp:TIBSQL;
 QBusca:TIBSQL;
 aindexname:string;
 afields:string;
 active,unique,descending:boolean;
 astatistics:variant;
 afieldcount:integer;
 procedure Appendrecord;
 begin
  dataset.append;
  try
   dataset.fieldbyname('INDEXNAME').AsString:=aindexname;
   dataset.fieldbyname('FIELDS').AsString:=afields;
   if unique then
    dataset.fieldbyname('UNIQUE').AsString:=SRpYes
   else
    dataset.fieldbyname('UNIQUE').AsString:=SRpNo;
   if active then
    dataset.fieldbyname('ACTIVE').AsString:=SRpYes
   else
    dataset.fieldbyname('ACTIVE').AsString:=SRpNo;
   if descending then
    dataset.fieldbyname('DESCENDING').AsString:=SRpYes
   else
    dataset.fieldbyname('DESCENDING').AsString:=SRpNo;
   dataset.fieldbyname('FIELDCOUNT').AsInteger:=afieldcount;
   dataset.fieldbyname('STATISTICS').Value:=astatistics;
   dataset.post;
  except
   dataset.cancel;
   raise;
  end;
 end;

begin
 dataset.disablecontrols;
 try
  Tran1:=TIBTransaction.create(Application);
  try
   tran1.DefaultDatabase:=base;
   QTemp:=TIBSQL.Create(Application);
   QBusca:=TIBSQL.Create(Application);
   try
    QTemp.database:=base;
    QTemp.transaction:=tran1;
    QBusca.database:=base;
    QBusca.transaction:=tran1;
    tran1.StartTransaction;
    try
     QTemp.SQL.text:='SELECT I.RDB$INDEX_NAME AINDEXNAME,'+
     ' S.RDB$FIELD_NAME AFIELDNAME,S.RDB$FIELD_POSITION APOSITION,'+
     ' I.RDB$UNIQUE_FLAG AUNIQUE,I.RDB$SEGMENT_COUNT AFIELDCOUNT, '+
     ' I.RDB$INDEX_INACTIVE AINACTIVE,I.RDB$INDEX_TYPE ATYPE,'+
     ' I.RDB$STATISTICS ASTATIST '+
     ' FROM RDB$INDICES I '+
     ' LEFT OUTER JOIN RDB$INDEX_SEGMENTS S '+
     ' ON I.RDB$INDEX_NAME=S.RDB$INDEX_NAME '+
     ' WHERE RDB$RELATION_NAME='''+table+''''+
     ' ORDER BY I.RDB$INDEX_NAME,S.RDB$FIELD_POSITION';
     QTemp.ExecQuery;
     try
      aindexname:='';
      afields:='';
      unique:=false;
      active:=false;
      astatistics:=0;
      afieldcount:=0;
      descending:=false;
      astatistics:=null;
      While Not QTemp.EOF do
      begin
       if aindexname<>Trim(QTemp.FieldByName('AINDEXNAME').AsString) then
       begin
        if aindexname<>'' then
        begin
         Appendrecord;
        end;
        aindexname:=Trim(QTemp.FieldByName('AINDEXNAME').AsString);
        afields:=Trim(QTemp.FieldByName('AFIELDNAME').AsString);
        afieldcount:=QTemp.FieldByName('AFIELDCOUNT').AsInteger;
        unique:=(QTemp.FieldByName('AUNIQUE').AsInteger=1);
        active:=(QTemp.FieldByName('AINACTIVE').AsInteger=0);
        descending:=(QTemp.FieldByName('ATYPE').AsInteger=1);
        astatistics:=QTemp.FieldByName('ASTATIST').Value;
       end
       else
       begin
        afields:=afields+','+Trim(QTemp.FieldByName('AFIELDNAME').AsString);
       end;

       QTemp.next;

       if assigned(fprogres)then
       begin
        fprogres.IncFetch(1);
       end;
      end;
      if aindexname<>'' then
      begin
       appendrecord;
      end;
     finally
      QTemp.CLose;
      QBusca.close;
     end;
     dataset.first;
    finally
     tran1.rollback;
    end;
   finally
    QTemp.free;
    QBusca.free;
   end;
  finally
   tran1.free;
  end;
 finally
  dataset.enablecontrols;
 end;
end;




procedure Filldatasetwithdomainsinfo(base:TIBDatabase;viewsystem:boolean;systemdomains:boolean;data:TDataset;fprogres:TFFetching);
var
 Tran1:TIBTransaction;
 QDomain:TIBSQL;
 QSearch:TIBSQL;
 dinfo:TIBDefDomain;
begin
 Tran1:=TIBTransaction.create(Application);
 try
  tran1.DefaultDatabase:=base;
  QDomain:=TIBSQL.Create(Application);
  QSearch:=TIBSQL.Create(Application);
  try
   QSearch.database:=base;
   QSearch.transaction:=tran1;
   QDomain.database:=base;
   QDomain.transaction:=tran1;
   tran1.StartTransaction;
   try
    QDomain.SQL.TExt:='SELECT RDB$FIELD_NAME DOMINI,RDB$VALIDATION_SOURCE VALIDA,RDB$COMPUTED_SOURCE CALCULA,'+
     'RDB$DEFAULT_SOURCE OMISION,RDB$FIELD_LENGTH TAMANY,RDB$SEGMENT_LENGTH SEGLEN,'+
     'RDB$FIELD_SCALE ESCALA,RDB$NULL_FLAG NONULO,'+
     'RDB$CHARACTER_SET_ID CHARSET,RDB$COLLATION_ID ORDRE,'+
      'RDB$FIELD_TYPE TIPO,RDB$FIELD_SUB_TYPE SUBTIPO,'+
     'RDB$DIMENSIONS DIMENSIO,RDB$CHARACTER_LENGTH CHARLEN'+
     ' FROM RDB$FIELDS ';
    if Not viewsystem then
     QDomain.SQL.TExt:=QDomain.SQL.TExt+' WHERE ( (RDB$SYSTEM_FLAG=0) or (RDB$SYSTEM_FLAG IS NULL) ) ';
    if Not Systemdomains then
    begin
     if Not viewsystem then
      QDomain.SQL.text:=QDomain.SQL.Text+' AND '
     else
      QDomain.SQL.text:=QDomain.SQL.Text+' WHERE ';
     QDomain.SQL.text:=QDomain.SQL.Text+' (NOT (RDB$FIELD_NAME LIKE ''RDB$%''))';
    end;
    QDomain.SQL.TExt:=QDomain.SQL.TExt+'ORDER BY 1';
    QDomain.ExecQuery;
    try
     While Not QDomain.EOF do
     begin
      dinfo:=FillDomainFromQuerys(QDomain,QSearch,base.sqldialect,fprogres);
      data.Append;
      try
       data.FieldByName('FIELDNAME').Value:=dinfo.name;
       data.FieldByName('DESCRIP').Value:=ibfieldtypetosource(dinfo.basetype,dinfo.size,dinfo.scale,base.sqldialect)
        +dinfo.arraysource;
       if dinfo.Notnull then
        data.FieldByName('NULLS').Value:=SRpNo
       else
        data.FieldByName('NULLS').Value:=SRpYes;
       data.FieldByName('CHECK').Value:=dinfo.check;
       data.FieldByName('DEFAULT').Value:=dinfo.default;
       data.FieldByName('COMPUTED').Value:=dinfo.computed;
       data.FieldByName('CHARACTER_SET').Value:=dinfo.character_set;
       data.FieldByName('COLLATION').Value:=dinfo.collation;
       data.FieldByName('SUBTYPE').Value:=dinfo.subtype;
       data.FieldByName('SEGSIZE').Value:=dinfo.segmentsize;
       data.FieldByName('SIZEBYTES').Value:=dinfo.sizebytes;
       data.post;
      except
       data.cancel;
       raise;
      end;
      QDomain.Next;
      if assigned(fprogres)then
      begin
       fprogres.IncFetch(1);
      end;
     end;
    finally
     QDomain.CLose;
    end;
    tran1.commitretaining;
   except
    tran1.rollback;
    raise;
   end;
  finally
   QSearch.free;
   QDomain.free;
  end;
 finally
  tran1.free;
 end;
end;

procedure FillCheckInfos(base:TIBDatabase;table:string;dataset:TDataset;fprogres:TFFetching);
var
 Tran1:TIBTransaction;
 QTemp:TIBSQL;
 QBusca:TIBSQL;
 defcon:TIBDefconstraint;
begin
 dataset.disablecontrols;
 try
 Tran1:=TIBTransaction.create(Application);
  try
   tran1.DefaultDatabase:=base;
   QTemp:=TIBSQL.Create(Application);
   QBusca:=TIBSQL.Create(Application);
   try
    QTemp.database:=base;
    QTemp.transaction:=tran1;
    QBusca.database:=base;
    QBusca.transaction:=tran1;
    tran1.StartTransaction;
    try
     QTemp.SQL.text:='SELECT RDB$CONSTRAINT_NAME CONNAME,'+
     ' RDB$CONSTRAINT_TYPE TIPO,RDB$INDEX_NAME INDICE '+
     'FROM RDB$RELATION_CONSTRAINTS WHERE RDB$RELATION_NAME='''+table+''''+
     ' ORDER BY 1,2';
     QTemp.ExecQuery;
     try
      While Not QTemp.EOF do
      begin
       defcon:=Fillconstraintfromquery(QTemp,QBusca,fprogres);
       // Assignem
       dataset.append;
       try
        dataset.fieldbyname('NAME').AsString:=defcon.name;
        dataset.fieldbyname('TYPE').AsString:=defcon.keyname;
        dataset.fieldbyname('FIELD').AsString:=defcon.fields;
        dataset.fieldbyname('UPDATE').AsString:=defcon.onupdate;
        dataset.fieldbyname('DELETE').AsString:=defcon.ondelete;
        dataset.fieldbyname('FIELDS').AsString:=defcon.destfields;
        dataset.fieldbyname('TABLE').AsString:=defcon.desttable;
        dataset.fieldbyname('DESCRIPTION').AsString:=defcon.checksource;
        dataset.fieldbyname('INDEX').AsString:=defcon.index;
        dataset.post;
       except
        dataset.cancel;
        raise;
       end;
       QTemp.next;
       if assigned(fprogres)then
       begin
        Application.processmessages;
        if fprogres.cancel then
         Raise Exception.Create(SRpOperationAborted);
       end;
      end;
     finally
      QTemp.CLose;
      QBusca.close;
     end;
     tran1.commitretaining;
     dataset.first;
    except
     tran1.rollback;
     raise;
    end;
   finally
    QTemp.free;
    QBusca.free;
   end;
  finally
   tran1.free;
  end;
 finally
  dataset.enablecontrols;
 end;
end;

function FillFieldinfo(base:TIBDatabase;table,field:string;fprogres:TFFetching):TIBDeffield;
var
 Tran1:TIBTransaction;
 QTemp:TIBSQL;
 QBusca:TIBSQL;
begin
 Result.name:=field;
 Tran1:=TIBTransaction.create(Application);
 try
  tran1.DefaultDatabase:=base;
  QTemp:=TIBSQL.Create(Application);
  QBusca:=TIBSQL.Create(Application);
  try
   QBusca.database:=base;
   QBusca.transaction:=tran1;
   QTemp.database:=base;
   QTemp.transaction:=tran1;
   tran1.StartTransaction;
   try
    QTemp.SQL.text:='SELECT RDB$FIELD_NAME CAMP,'+
    ' RDB$FIELD_SOURCE SOURCE,RDB$FIELD_POSITION POSICIO,'+
    ' RDB$DEFAULT_SOURCE DEFVALUE,RDB$NULL_FLAG NULO,'+
    ' RDB$SECURITY_CLASS,RDB$COLLATION_ID ORDRE'+
    ' FROM RDB$RELATION_FIELDS '+
    ' WHERE RDB$RELATION_NAME='''+table+''''+
    ' AND RDB$FIELD_NAME='''+field+'''';
    QTemp.ExecQuery;
    try
     if Not (QTemp.BOF AND QTemp.EOF) then
     begin
      Result.defdomain:=FillDomaininfo(base,QTemp.fieldbyname('SOURCE').AsString,fprogres);
//      if pos('RDB$',Result.defdomain.name)=1 then
//       Result.defdomain.name:='';
      if QTemp.fieldbyname('DEFVALUE').Isnull then
      begin
       Result.default:=REsult.defdomain.default;
      end
      else
       if Length(QTemp.fieldbyname('DEFVALUE').AsString)>0 then
       begin
        Result.default:=QTemp.fieldbyname('DEFVALUE').AsString;
       end
       else
        Result.default:=result.defdomain.default;
      if QTemp.fieldbyname('NULO').isnull then
       Result.notnull:=REsult.defdomain.notnull
      else
       Result.notnull:=(QTemp.fieldbyname('NULO').ASInteger=1);
      Result.order:=QTemp.fieldbyname('POSICIO').Asinteger;
       try
         if Not QTemp.FieldByName('ORDRE').isnull then
         begin
          QBusca.Close;
          QBusca.SQL.TExt:='SELECT RDB$COLLATION_NAME NOM '+
          'FROM RDB$COLLATIONS WHERE RDB$CHARACTER_SET_ID='+
          IntToStr(Result.defdomain.charsetid)+' AND RDB$COLLATION_ID='+
          QTemp.FieldByName('ORDRE').AsString;
          QBusca.ExecQuery;
          if (QBusca.EOF AND QBusca.BOF) then
          begin
           Result.collation:='';
          end
          else
          begin
           Result.Collation:=QBusca.fieldbyname('NOM').Value;
          end;
         end
         else
         begin
          Result.collation:='';
         end;
       finally
        QBusca.Close;
       end;
//       if Pos('RDB$',Result.defdomain.name)=1 then
//        Result.defdomain.name:='';
     end;
    finally
     QTemp.CLose;
    end;
    tran1.commitretaining;
   except
    tran1.rollback;
    raise;
   end;
  finally
   QBusca.free;
   QTemp.free;
  end;
 finally
  tran1.free;
 end;
end;

function Filldatasetwithfieldsinfo(base:TIBDatabase;viewsystem:boolean;data:TDataset;table:string;fprogres:TFFetching):integer;
var
 Tran1:TIBTransaction;
 QDomain:TIBSQL;
 QSearch:TIBSQL;
 dinfo:TIBDefDomain;
begin
 Result:=0;
 Tran1:=TIBTransaction.create(Application);
 try
  tran1.DefaultDatabase:=base;
  QDomain:=TIBSQL.Create(Application);
  QSearch:=TIBSQL.Create(Application);
  try
   QSearch.database:=base;
   QSearch.transaction:=tran1;
   QDomain.database:=base;
   QDomain.transaction:=tran1;
   tran1.StartTransaction;
   try
    QDomain.SQL.TExt:='SELECT D.RDB$FIELD_NAME DOMINI,D.RDB$VALIDATION_SOURCE VALIDA,D.RDB$COMPUTED_SOURCE CALCULA,'+
     'D.RDB$DEFAULT_SOURCE OMISION,D.RDB$FIELD_LENGTH TAMANY,D.RDB$SEGMENT_LENGTH SEGLEN,'+
     'D.RDB$FIELD_SCALE ESCALA,D.RDB$NULL_FLAG NONULO,'+
     'D.RDB$CHARACTER_SET_ID CHARSET,D.RDB$COLLATION_ID ORDRE,'+
      'D.RDB$FIELD_TYPE TIPO,D.RDB$FIELD_SUB_TYPE SUBTIPO,'+
     'D.RDB$DIMENSIONS DIMENSIO,D.RDB$CHARACTER_LENGTH CHARLEN,'+
     'F.RDB$FIELD_NAME,F.RDB$FIELD_POSITION,F.RDB$DEFAULT_SOURCE,'+
     ' F.RDB$NULL_FLAG,F.RDB$COLLATION_ID ORDRE1'+
     ' FROM RDB$RELATION_FIELDS F'+
     ' LEFT OUTER JOIN RDB$FIELDS D '+
     ' ON F.RDB$FIELD_SOURCE=D.RDB$FIELD_NAME '+
     ' WHERE RDB$RELATION_NAME='''+table+''' ';
    if Not viewsystem then
     QDomain.SQL.TExt:=QDomain.SQL.TExt+' AND ( (RDB$SYSTEM_FLAG=0) or (RDB$SYSTEM_FLAG IS NULL) ) ';
    QDomain.SQL.TExt:=QDomain.SQL.TExt+' ORDER BY F.RDB$FIELD_POSITION';
    QDomain.ExecQuery;
    try
     While Not QDomain.EOF do
     begin
      dinfo:=FillDomainFromQuerys(QDomain,QSearch,base.sqldialect,fprogres);
      data.Append;
      try
       data.FieldByName('FIELD').Value:=QDomain.FieldByName('RDB$FIELD_NAME').AsString;
       data.FieldByName('ORDER').Value:=QDomain.FieldByName('RDB$FIELD_POSITION').AsInteger;
       data.FieldByName('FIELDNAME').Value:=dinfo.name;
       data.FieldByName('DESCRIP').Value:=ibfieldtypetosource(dinfo.basetype,dinfo.size,dinfo.scale,base.sqldialect)
        +dinfo.arraysource;
       if dinfo.Notnull then
       begin
        data.FieldByName('NULLS').Value:=SRpNo;
       end
       else
       begin
        if QDomain.FieldByName('RDB$NULL_FLAG').Value=1 then
        begin
         data.FieldByName('NULLS').Value:=SRpNo;
        end
        else
        begin
         data.FieldByName('NULLS').Value:=SRpYes
        end;
       end;
       data.FieldByName('CHECK').Value:=dinfo.check;
       if Not QDomain.fieldbyname('RDB$DEFAULT_SOURCE').isnull then
        data.FieldByName('DEFAULT').Value:=QDomain.fieldbyname('RDB$DEFAULT_SOURCE').AsString
       else
        data.FieldByName('DEFAULT').Value:=dinfo.default;
       data.FieldByName('COMPUTED').Value:=dinfo.computed;
       data.FieldByName('CHARACTER_SET').Value:=dinfo.character_set;
       data.FieldByName('COLLATION').Value:=dinfo.collation;
       data.FieldByName('SUBTYPE').Value:=dinfo.subtype;
       data.FieldByName('SEGSIZE').Value:=dinfo.segmentsize;
       data.FieldByName('SIZEBYTES').Value:=dinfo.sizebytes;
       if dinfo.sizebytes<>null then
        Result:=Result+dinfo.sizebytes;
       data.post;
      except
       data.cancel;
       raise;
      end;
      QDomain.Next;
      if assigned(fprogres)then
      begin
       fprogres.IncFetch(1);
      end;
     end;
    finally
     QDomain.CLose;
    end;
    tran1.commitretaining;
   except
    tran1.rollback;
    raise;
   end;
  finally
   QSearch.free;
   QDomain.free;
  end;
 finally
  tran1.free;
 end;
end;

function TriggerTypetostring(atype:integer):string;
begin
 case atype of
  1:
   begin
    Result:='BEFORE INSERT';
   end;
  2:
   begin
    Result:='AFTER INSERT';
   end;
  3:
   begin
    Result:='BEFORE UPDATE';
   end;
  4:
   begin
    Result:='AFTER UPDATE';
   end;
  5:
   begin
    Result:='BEFORE DELETE';
   end;
  6:
   begin
    Result:='AFTER DELETE';
   end;
  else
   begin
    Result:=SrpUnknown;
   end;
 end;
end;

procedure FillTriggerInfos(base:TIBDatabase;table:string;dataset:TDataset;fprogres:TFFetching);
var
 Tran1:TIBTransaction;
 QTemp:TIBSQL;
begin
 dataset.disablecontrols;
 try
  Tran1:=TIBTransaction.create(Application);
  try
   tran1.DefaultDatabase:=base;
   QTemp:=TIBSQL.Create(Application);
   try
    QTemp.database:=base;
    QTemp.transaction:=tran1;
    tran1.StartTransaction;
    try
     QTemp.SQL.text:='SELECT RDB$TRIGGER_NAME,RDB$TRIGGER_TYPE,'+
     ' RDB$TRIGGER_SEQUENCE,RDB$TRIGGER_INACTIVE,RDB$FLAGS,'+
     ' RDB$TRIGGER_SOURCE '+
     ' FROM RDB$TRIGGERS '+
     ' WHERE RDB$RELATION_NAME='''+table+''''+
     ' AND (NOT (RDB$TRIGGER_SOURCE IS NULL)) '+
     ' ORDER BY RDB$TRIGGER_TYPE,RDB$TRIGGER_SEQUENCE';
     QTemp.ExecQuery;
     try
      While Not QTemp.EOF do
      begin
       dataset.append;
       try
        dataset.fieldbyname('NAME').AsString:=QTemp.fieldbyname('RDB$TRIGGER_NAME').AsString;
        dataset.fieldbyname('TYPEINT').AsInteger:=QTemp.fieldbyname('RDB$TRIGGER_TYPE').AsInteger;
        dataset.fieldbyname('TYPE').AsString:=TriggerTypeToString(QTemp.fieldbyname('RDB$TRIGGER_TYPE').AsInteger);
        dataset.fieldbyname('POSITION').AsInteger:=QTemp.fieldbyname('RDB$TRIGGER_SEQUENCE').AsInteger;
        dataset.fieldbyname('FLAGS').AsInteger:=QTemp.fieldbyname('RDB$FLAGS').AsInteger;
        dataset.fieldbyname('SOURCE').AsString:=QTemp.fieldbyname('RDB$TRIGGER_SOURCE').AsString;
        if QTemp.FieldByName('RDB$TRIGGER_INACTIVE').AsInteger=0 then
        begin
         dataset.fieldbyname('ACTIVE').AsString:=SRpYes;
         dataset.fieldbyname('ACTIVEBOOL').Asboolean:=True;
        end
        else
        begin
         dataset.fieldbyname('ACTIVE').AsString:=SRpNo;
         dataset.fieldbyname('ACTIVEBOOL').Asboolean:=False;
        end;
        dataset.post;
       except
        dataset.cancel;
        raise;
       end;
       if assigned(fprogres)then
       begin
        fprogres.IncFetch(1);
       end;
       QTemp.next;
      end;
     finally
      QTemp.CLose;
     end;
     dataset.first;
    finally
     tran1.rollback;
    end;
   finally
    QTemp.free;
   end;
  finally
   tran1.free;
  end;
 finally
  dataset.enablecontrols;
 end;
end;

procedure FillWithUsers(user,password,server:string;protocol:TrpIBXProtocol;alist:TStrings);
var
 ibsec:TIBSecurityService;
 i:integer;
 atatched:boolean;
begin
 alist.clear;
 ibsec:=TIBSecurityService.Create(Application);
 try
  ibsec.Params.clear;
  ibsec.params.add('user_name='+user);
  ibsec.params.add('password='+password);
  ibsec.ServerName:=server;
  if Length(ibsec.servername)>0 then
  begin
   case protocol of
    rpIBXTCP:
     ibsec.Protocol:=TCP;
    rpIBXNetBeui:
     ibsec.protocol:=NamedPipe;
    rpIBXIpx:
     ibsec.protocol:=SPX;
   end;
  end
  else
   ibsec.Protocol:=Local;
  IBSec.loginprompt:=false;
  atatched:=true;
  try
   IBSec.Attach;
  except
   atatched:=false;
  end;
  if not atatched then
   exit;
  try
   IBSec.DisplayUsers;
   for i:=0 to IBSec.UserInfoCount-1 do
   begin
    alist.add(IBSec.UserInfo[i].UserName);
   end;
  finally
   try
    IBSec.Detach;
   except
   end;
  end;
 finally
  ibsec.free;
 end;
end;


end.
