{*******************************************************}
{                                                       }
{       IBAccess                                        }
{       Metadata extraction dialog                      }
{                                                       }
{       Author:                                         }
{       Toni Martir                                     }
{       http://www.ibaccess.org                         }
{       e-mail: toni@pala.com                           }
{                                                       }
{       OpenSource license (MPL)                        }
{                                                       }
{*******************************************************}

unit metaextract;

interface

uses
  SysUtils, Classes, QForms,
  IBDatabase, IBExtract, 
  QMenus, IBSQL, QActnList, QDialogs, QImgList, QTypes, QStdCtrls,
  QButtons, QExtCtrls, QComCtrls, QControls;

type
  TFMetaextract = class(TForm)
    Memo: TMemo;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    ASave1: TMenuItem;
    N1: TMenuItem;
    Close1: TMenuItem;
    Edit1: TMenuItem;
    ACopy1: TMenuItem;
    IconImages: TImageList;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    SaveDialog1: TSaveDialog;
    ActionList1: TActionList;
    ASave: TAction;
    AClose: TAction;
    ACopy: TAction;
    Panel1: TPanel;
    Label1: TLabel;
    LObject: TLabel;
    ComboExtract: TComboBox;
    ComboObject: TComboBox;
    BExtract: TBitBtn;
    IBExtract1: TIBExtract;
    Trans: TIBTransaction;
    Checkshowsystem: TCheckBox;
    IBSQL1: TIBSQL;
    procedure ACopyExecute(Sender: TObject);
    procedure ASaveExecute(Sender: TObject);
    procedure ACloseExecute(Sender: TObject);
    procedure BExtractClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ComboExtractClick(Sender: TObject);
    procedure ComboObjectDropDown(Sender: TObject);
    procedure CheckshowsystemClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
    validobjects:boolean;
    FBase:TIBDatabase;
    olddefault:TIBTransaction;
    procedure Setbase(value:Tibdatabase);
  public
    { Public declarations }
    property base:TIBDatabase read Fbase write Setbase;
  end;

procedure ShowIBExtract(base:TIBDatabase);

implementation

{$R *.xfm}

procedure ShowIBExtract(base:TIBDatabase);
var
 dia:TFMetaExtract;
begin
 dia:=TFMetaExtract.Create(Application);
 try
  dia.base:=base;
  dia.showmodal;
 finally
  dia.free;
 end;
end;

procedure TFMetaextract.ACopyExecute(Sender: TObject);
begin
 Memo.CopyToClipboard;
end;

procedure TFMetaextract.ASaveExecute(Sender: TObject);
begin
 if savedialog1.Execute then
 begin
  Memo.Lines.SaveToFile(savedialog1.filename);
 end;
end;

procedure TFMetaextract.ACloseExecute(Sender: TObject);
begin
 Close;
end;

procedure TFMetaextract.Setbase(value:Tibdatabase);
begin
 fbase:=Value;
 trans.defaultdatabase:=value;
 ibextract1.transaction:=trans;
 ibextract1.database:=value;
 ibsql1.database:=value;
 if assigned(value) then
  olddefault:=value.defaulttransaction;
 fbase.defaulttransaction:=trans;
end;


procedure TFMetaextract.BExtractClick(Sender: TObject);
begin
 trans.starttransaction;
 try
  IBExtract1.ShowSystem:=Checkshowsystem.Checked;
  if Length(Trim(ComboObject.Text))>0 then
  begin
   IBExtract1.ExtractObject(TExtractObjectTypes(ComboExtract.itemindex),
    ComboObject.text);
  end
  else
   IBExtract1.ExtractObject(TExtractObjectTypes(ComboExtract.itemindex));
 finally
  trans.rollback;
 end;
 Memo.lines.assign(IBExtract1.Items);
 ACopy.enabled:=true;
 ASave.enabled:=true;
end;

procedure TFMetaextract.FormCreate(Sender: TObject);
begin
 ComboExtract.itemindex:=0;
end;

procedure TFMetaextract.ComboExtractClick(Sender: TObject);
begin
 ComboObject.text:='';
 LObject.visible:=ComboExtract.itemindex>0;
 ComboObject.visible:=ComboExtract.itemindex>0;
 validobjects:=false;
end;

procedure TFMetaextract.ComboObjectDropDown(Sender: TObject);
var
 astring:string;
begin
 if validobjects then
  exit;
 ComboObject.items.clear;
 astring:='';
 Trans.starttransaction;
 try
  case ComboExtract.itemindex of
   1:
    begin
     astring:='SELECT RDB$FIELD_NAME FROM RDB$FIELDS ';
     if Not Checkshowsystem.checked then
      astring:=astring+' WHERE ((RDB$SYSTEM_FLAG IS NULL) OR (RDB$SYSTEM_FLAG=0))';
    end;
   2,14:
    begin
     astring:='SELECT RDB$RELATION_NAME FROM RDB$RELATIONS '+
      ' WHERE (RDB$VIEW_SOURCE IS NULL)';
     if Not Checkshowsystem.checked then
      astring:=astring+' AND ((RDB$SYSTEM_FLAG IS NULL) OR (RDB$SYSTEM_FLAG=0))';
    end;
   3:
    begin
     astring:='SELECT RDB$RELATION_NAME FROM RDB$RELATIONS '+
      ' WHERE (NOT (RDB$VIEW_SOURCE IS NULL))';
     if Not Checkshowsystem.checked then
      astring:=astring+' AND ((RDB$SYSTEM_FLAG IS NULL) OR (RDB$SYSTEM_FLAG=0))';
    end;
   4:
    begin
     astring:='SELECT RDB$PROCEDURE_NAME FROM RDB$PROCEDURES ';
     if Not Checkshowsystem.checked then
      astring:=astring+' WHERE ((RDB$SYSTEM_FLAG IS NULL) OR (RDB$SYSTEM_FLAG=0))';
    end;
   5:
    begin
     astring:='SELECT RDB$FUNCTION_NAME FROM RDB$FUNCTIONS ';
    end;
   6:
    begin
     astring:='SELECT RDB$GENERATOR_NAME FROM RDB$GENERATORS ';
     if Not Checkshowsystem.checked then
      astring:=astring+' WHERE ((RDB$SYSTEM_FLAG IS NULL) OR (RDB$SYSTEM_FLAG=0))';
    end;
   7:
    begin
     astring:='SELECT RDB$EXCEPTION_NAME FROM RDB$EXCEPTIONS ';
     if Not Checkshowsystem.checked then
      astring:=astring+' WHERE ((RDB$SYSTEM_FLAG IS NULL) OR (RDB$SYSTEM_FLAG=0))';
    end;
   8:
    begin
     astring:='SELECT RDB$FUNCTION_NAME FROM RDB$FILTERS ';
     if Not Checkshowsystem.checked then
      astring:=astring+' WHERE ((RDB$SYSTEM_FLAG IS NULL) OR (RDB$SYSTEM_FLAG=0))';
    end;
   9:
    begin
     astring:='SELECT RDB$ROLE_NAME FROM RDB$ROLES ';
    end;
   10:
    begin
     astring:='SELECT RDB$TRIGGER_NAME FROM RDB$TRIGGERS ';
     if Not Checkshowsystem.checked then
      astring:=astring+' WHERE ((RDB$SYSTEM_FLAG IS NULL) OR (RDB$SYSTEM_FLAG=0))';
    end;
   11:
    begin
     astring:='SELECT RDB$CONSTRAINT_NAME ' +
     'FROM RDB$RELATION_CONSTRAINTS ' +
     ' WHERE RDB$CONSTRAINT_TYPE = ''FOREIGN KEY''';
    end;
   12:
    begin
     astring:='SELECT RDB$INDEX_NAME FROM RDB$INDICES ';
     if Not Checkshowsystem.checked then
      astring:=astring+' WHERE ((RDB$SYSTEM_FLAG IS NULL) OR (RDB$SYSTEM_FLAG=0))';
    end;
   13:
    begin
     astring:='SELECT RDB$CONSTRAINT_NAME FROM RDB$CHECK_CONSTRAINTS';
    end;
  end;
  astring:=astring+' ORDER BY 1';
  IBSQL1.SQL.Text:=astring;
  IBSQL1.Execquery;
  While Not IBSQL1.EOF do
  begin
   ComboObject.items.Add(IBSQL1.fields[0].Asstring);
   IBSQL1.Next;
  end;
 finally
  Trans.commit;
 end;
end;

procedure TFMetaextract.CheckshowsystemClick(Sender: TObject);
begin
 validobjects:=false;
end;

procedure TFMetaextract.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
 if assigned(fbase) then
 begin
  fbase.defaulttransaction:=olddefault;
 end;
end;

end.
