{*******************************************************}
{                                                       }
{       IBAccess                                        }
{       New database form                               }
{                                                       }
{       Author:                                         }
{       Toni Martir                                     }
{       http://www.ibaccess.org                         }
{       e-mail: toni@pala.com                           }
{                                                       }
{       OpenSource license (MPL)                        }
{                                                       }
{*******************************************************}

unit newdatabase;

interface

uses
  SysUtils, Classes, QForms,variants,
  Db, IBDatabase, ibacconst,
  rpibdb, DBClient, QDialogs, QButtons, QStdCtrls, QControls, QGrids,
  QDBGrids;

type
  TFNewDatabase = class(TForm)
    EPassword: TEdit;
    Label3: TLabel;
    Label2: TLabel;
    EUsername: TEdit;
    Label4: TLabel;
    ComboChar: TComboBox;
    Label6: TLabel;
    BCancel: TButton;
    BOK: TButton;
    Label5: TLabel;
    ComboPage: TComboBox;
    DataSource1: TDataSource;
    Label7: TLabel;
    DBGrid1: TDBGrid;
    SaveDialog1: TSaveDialog;
    Label1: TLabel;
    DMemtable: TClientdataset;
    DMemtableFile: TStringField;
    DMemtableSize: TIntegerField;
    DMemtableSizeMb: TIntegerField;
    EServer: TEdit;
    LProtocol: TLabel;
    ComboProtocol: TComboBox;
    ComboVersion: TComboBox;
    Label10: TLabel;
    GConf: TGroupBox;
    BCOnfig: TBitBtn;
    ComboAlias: TComboBox;
    LAlias: TLabel;
    Erole: TEdit;
    Label8: TLabel;
    procedure FormShow(Sender: TObject);
    procedure BOKClick(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure ComboPageChange(Sender: TObject);
    procedure DMemtableCalcFields(DataSet: TDataSet);
    procedure EServerChange(Sender: TObject);
    procedure BCOnfigClick(Sender: TObject);
    procedure ComboAliasClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    procedure Createbase;
  public
    { Public declarations }
    base:TIBDatabase;
    created:boolean;
    filename:string;
    protocol:TrpIBXProtocol;
    server:string;
    dbconf:TRpDBIBconf;
  end;


function CreateNewdatabase(dbase:TIBDatabase;var server,filename,username,password,charset,role:string;var protocol:TrpIBXProtocol;dbconf:TRpDBIBconf):boolean;

implementation

{$R *.xfm}

procedure TFNewDatabase.FormShow(Sender: TObject);
begin
 if Assigned(dbconf) then
 begin
  GConf.visible:=True;
  comboalias.Items.Assign(dbconf.databases);
 end;
 if combochar.itemindex<0 then
  ComboChar.itemindex:=Combochar.items.indexof('WIN1252');
 ComboPage.Itemindex:=3;
 comboversion.itemindex:=1;
 ComboProtocol.Itemindex:=0;
 DMemTable.close;
 DMemTable.createdataset;
end;

procedure TFNewDatabase.BOKClick(Sender: TObject);
begin
 DMemTable.checkbrowsemode;
 Createbase;
 created:=True;
 close;
end;

procedure TFNewDatabase.FormHide(Sender: TObject);
begin
 DMemTable.Close;
end;

procedure TFNewDatabase.Createbase;
var
 size:integer;
begin
 base.Params.clear;
 case comboversion.itemindex of
  0:
   base.SQLdialect:=1;
  else
  begin
   base.SQLdialect:=3;
  end;
 end;
 base.params.add(' USER '''+EUsername.text+''' PASSWORD '''+EPassword.text+'''');
 base.Params.Add(' PAGE_SIZE '+Combopage.text);
 DMemTable.disablecontrols;
 try
  if (DMemTable.BOF AND DMemTable.EOF) then
   raise Exception.create(SRpAtLeastOne);
  // Comprovem que tots tenen size excepte l'ultim
  DMemTable.first;
  While not DMemTable.eof do
  begin
   if DMemTablesize.IsNull then
    break;
   DMemTable.next;
  end;
  if DMemTable.eof then
  begin
   DMemTable.Edit;
   try
    DMemTablesize.asvariant:=null;
    DMemTable.posT;
   except
    DMemTable.cancel;
    Raise;
   end;
  end
  else
  begin
   DMemTable.next;
   If not DMemTable.eof then
    Raise Exception.create(SRpSpecsize);
  end;
  DMemTable.first;
  filename:=DMemTablefile.asstring;
  server:=EServer.text;
  protocol:=TrpIBXProtocol(COmboProtocol.ItemIndex);
  base.databasename:=BuildIBDatabaseName((Length(Trim(EServer.text))>0),TRpIBXProtocol(ComboProtocol.ItemIndex),EServer.Text,DMemTablefile.asstring);


  if Not DMemTablesize.isnull then
   size:=DMemTablesize.asinteger
  else
   size:=0;
  DMemTable.next;
  if Not DMemTable.eof then
   base.Params.add(' LENGTH '+IntToStr(size));
  base.Params.Add(' DEFAULT CHARACTER SET '+Combochar.text);
  While Not DMemTable.eof do
  begin
   if size<0 then
    Raise Exception.Create(SRpIncorrectsize);
   base.Params.Add(' FILE '''+DMemTablefile.asstring+'''');
   DMemTable.next;
   if Not DMemTableSize.isnull then
    size:=DMemTablesize.asinteger
   else
    size:=0;
   if Not DMemTable.eof then
    base.Params.add(' LENGTH '+IntToStr(size));
  end;
  base.CreateDatabase;
  base.connected:=false;
  base.Params.clear;
  base.params.add('user_name='+Eusername.text);
  base.params.add('password='+Epassword.Text);
  if Length(Trim(Erole.text))>0 then
   base.params.add('sql_role_name='+Erole.text);
  base.params.add('lc_ctype='+ComboChar.text);
 finally
  DMemTable.enablecontrols;
 end;
end;

procedure TFNewDatabase.ComboPageChange(Sender: TObject);
begin
 DMemTable.disablecontrols;
 try
  DMemTable.first;
  While Not DmemTable.eof do
  begin
   DMemTable.edit;
   try
    DMemTablesize.asvariant:=DMemtablesize.asvariant;
    DMemTable.post;
   except
    DMemTable.cancel;
    raise;
   end;
   DMemTable.Next;
  end;
 finally
  DMemTable.enablecontrols;
 end;
end;

function CreateNewdatabase(dbase:TIBDatabase;var server,filename,username,password,charset,role:string;var protocol:TrpIBXProtocol;dbconf:TRpDBIBconf):boolean;
var
 dia:TFNewDatabase;
begin
 dia:=TFNewdatabase.create(Application);
 try
  dia.base:=dbase;
  dia.EUsername.Text:=username;
  dia.EPassword.Text:=password;
  dia.ERole.text:=role;
  dia.dbconf:=dbconf;
  dia.ComboChar.itemindex:=dia.combochar.items.indexof(charset);
  dia.showmodal;
  Result:=dia.created;
  if Result then
  begin
   username:=dia.EUsername.text;
   password:=dia.EPassword.text;
   charset:=dia.ComboChar.text;
   server:=dia.server;
   filename:=dia.filename;
   protocol:=dia.protocol;
   role:=dia.ERole.text;
  end;
 finally
  dia.free;
 end;
end;

procedure TFNewDatabase.DMemtableCalcFields(DataSet: TDataSet);
begin
  if DMemTableSize.isnull then
  DMemTableSizeMB.asvariant:=null
 else
  DMemTableSizeMb.asinteger:=DMemTableSize.AsInteger*(StrToInt(ComboPage.items[ComboPage.itemindex]) div 1024) div 1024;
end;

procedure TFNewDatabase.EServerChange(Sender: TObject);
begin
 if Length(Trim(Eserver.Text))>0 then
 begin
  LProtocol.Visible:=True;
  ComboProtocol.Visible:=True;
 end
 else
 begin
  LProtocol.Visible:=False;
  ComboProtocol.Visible:=False;
 end;
end;

procedure TFNewDatabase.BCOnfigClick(Sender: TObject);
begin
 dbconf.ShowConfig(True,True,true);
 dbconf.LoadAliases;
 comboalias.Items.Assign(dbconf.databases);
 comboalias.itemindex:=-1;
end;

procedure TFNewDatabase.ComboAliasClick(Sender: TObject);
var
 dbinfo:TRpDBIBinfo;
begin
 if Comboalias.itemindex<0 then
  exit;
 dbinfo:=dbconf.databases.items[comboalias.itemindex];
 ComboProtocol.Itemindex:=integer(dbinfo.protocol);
 if dbinfo.SQLDialect=1 then
  ComboVersion.Itemindex:=0
 else
  ComboVersion.Itemindex:=1;
 EServer.text:=dbinfo.Server;
 EserverChange(Self);
 if (DMemtable.state in dseditmodes) then
  DMemtable.cancel;
 DMemTable.Close;
 DMemTable.Createdataset;
 DMemTable.insert;
 DMemtableFile.Value:=dbinfo.Filename;
 EUsername.text:=dbinfo.username;
 EPassword.text:=dbinfo.password;
 ComboChar.itemindex:=Combochar.items.indexof(dbinfo.CharacterSet);
end;

procedure TFNewDatabase.FormCreate(Sender: TObject);
begin
 DMemtable.CreateDataSet;
end;

end.
