{*******************************************************}
{                                                       }
{       IBAccess                                        }
{       Declare function dialog                         }
{                                                       }
{       Author:                                         }
{       Toni Martir                                     }
{       http://www.ibaccess.org                         }
{       e-mail: toni@pala.com                           }
{                                                       }
{       OpenSource license (MPL)                        }
{                                                       }
{*******************************************************}

unit newfunction;

interface

uses Classes, QForms,SysUtils,variants,
  ibdatabase, Db,
  ibutil2, QMask, DBClient, QControls, QGrids, QDBGrids, QStdCtrls;

type
  TFNewFunction = class(TForm)
    BOK: TButton;
    CancelBtn: TButton;
    Label1: TLabel;
    EModule: TEdit;
    Label2: TLabel;
    EEntrypoint: TEdit;
    EName: TEdit;
    Label5: TLabel;
    Label3: TLabel;
    DataSource1: TDataSource;
    ComboType: TComboBox;
    Label4: TLabel;
    ESize: TMaskEdit;
    lSize: TLabel;
    Checkfreeit: TCheckBox;
    Label6: TLabel;
    DInput: TClientdataset;
    DInputTYPE: TStringField;
    DInputSIZE: TIntegerField;
    DInputDECIMALS: TIntegerField;
    DInputCHARSET: TStringField;
    GridInput: TDBGrid;
    EDecimals: TMaskEdit;
    LDecimals: TLabel;
    LCharset: TLabel;
    ComboCharset: TComboBox;
    procedure BOKClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormShow(Sender: TObject);
    procedure ComboTypeClick(Sender: TObject);
    procedure DInputTYPEChange(Sender: TField);
    procedure DInputAfterScroll(DataSet: TDataSet);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    docreate:boolean;
    fbase:TIBDatabase;
    procedure Createfunc;
    procedure SetBase(value:TIBDatabase);
  public
    { Public declarations }
    property base:TIbDatabase read fbase write Setbase;
  end;

procedure IBDropFunction(base:TIBDatabase;func:string);
function IBNewFunction(base:TIBDatabase):boolean;

implementation

{$R *.xfm}

procedure IBDropFunction(base:TIBDatabase;func:string);
begin
 IBExecuteSQL(base,'DROP EXTERNAL FUNCTION "'+func+'"');
end;

function IBNewFunction(base:TIBDatabase):boolean;
var
 dia:TFNewFunction;
begin
 dia:=TFNewFunction.create(Application);
 try
  dia.base:=base;
  Result:=dia.showmodal=mrok;
 finally
  dia.free;
 end;
end;

procedure TFNewFunction.BOKClick(Sender: TObject);
begin
 docreate:=true;
end;

procedure TFNewFunction.Createfunc;
var
 astring:string;
 params:string;
 amark:Tbookmark;
 adeli:string;
begin
 adeli:='';
 if base.sqldialect=3 then
  adeli:='"';
 DInput.CheckBrowsemode;
 astring:='DECLARE EXTERNAL FUNCTION '+adeli+EName.text+adeli+' ';
 params:='';
 if Not (DInput.BOF AND DInput.EOF) then
 begin
  DInput.disablecontrols;
  try
   amark:=DInput.getbookmark;
   try
    DInput.First;
    While Not DInput.Eof do
    begin
     if params='' then
      params:=params+DInputTYPE.Value
     else
      params:=params+' , '+DInputTYPE.Value;
     // Input size?, charset?
     if Uppercase(Dinputtype.AsString)='NUMERIC' then
     begin
      params:=params+'('+ESize.text+','+EDecimals.text+')';
     end
     else
     if ( (Uppercase(Dinputtype.AsString)='CHAR') OR
      (Uppercase(Dinputtype.AsString)='VARCHAR') OR
      (Uppercase(Dinputtype.AsString)='CSTRING') )
      then
     begin
      params:=params+'('+DInputsize.asstring+')';
      if DInputCHARSET.text<>'' then
       params:=params+' CHARACTER SET '+DInputCHARSET.AsString;
     end;
     DInput.Next;
    end;
    DInput.gotobookmark(amark);
   finally
    DInput.FreeBookmark(amark);
   end;
  finally
   DInput.enablecontrols;
  end;
 end;
 astring:=astring+params+' RETURNS ';
 // Return type
 case combotype.itemindex of
  4:
   begin
    astring:=astring+' NUMERIC('+ESize.text+','+EDecimals.text+')';
   end;
  6:
   begin
    astring:=astring+' CHAR('+ESize.text+')'+' CHARACTER SET '+ComboCharset.text;
   end;
  7:
   begin
    astring:=astring+' VARCHAR('+ESize.text+')'+' CHARACTER SET '+ComboCharset.text;
   end;
  8:
   begin
    astring:=astring+' CSTRING('+ESize.text+')'+' CHARACTER SET '+ComboCharset.text;
   end;
  else
   begin
    astring:=astring+ComboType.text;
   end;
 end;
 // Entry point
 if Checkfreeit.checked then
  astring:=astring+' FREE_IT ';
 astring:=astring+' ENTRY_POINT '+QuotedStr(EEntrypoint.text);
 astring:=astring+' MODULE_NAME '+QuotedStr(EModule.text);
 ibutil2.IBExecuteSQL(base,astring);
end;

procedure TFNewFunction.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
 try
  if docreate then
   Createfunc;
 finally
  docreate:=false;
 end;
end;

procedure TFNewFunction.SetBase(value:TIBDatabase);
begin
 fbase:=value;
 // For blobs no character set
end;

procedure TFNewFunction.FormShow(Sender: TObject);
var
 list:TStringlist;
begin
 if not assigned(base) then
  exit;
 list:=TStringlist.create;
 try
  list.add('SMALLINT');
  list.add('INTEGER');
  list.add('FLOAT');
  list.add('DOUBLE PRECISION');
  list.add('NUMERIC');
  list.add('DATE');
  list.add('CHAR');
  list.add('VARCHAR');
  if base.sqldialect=3 then
  list.add('CSTRING');
  list.add('BLOB');
  begin
   list.add('TIME');
   list.add('TIMESTAMP');
  end;
  gridinput.columns[0].PickList.assign(list);
  ComboType.items.assign(list);
  ComboType.itemindex:=1;
  ComboTypeClick(self);
  list.clear;
  list.add('NONE');
  list.add('OCTECTS');
  list.add('ASCII');
  list.add('UNICODE_FSS');
  list.add('SJIS_0208');
  list.add('EUCJ_0208');
  list.add('DOS437');
  list.add('DOS850');
  list.add('DOS865');
  list.add('ISO8859_1');
  list.add('DOS852');
  list.add('DOS857');
  list.add('DOS860');
  list.add('DOS861');
  list.add('DOS863');
  list.add('CYRL');
  list.add('WIN1250');
  list.add('WIN1251');
  list.add('WIN1252');
  list.add('WIN1253');
  list.add('WIN1254');
  list.add('NEXT');
  list.add('KSC_5601');
  list.add('BIG_5');
  list.add('GB_2312');
  ComboCharset.items.Assign(list);
  ComboCharset.itemindex:=0;
 finally
  list.free;
 end;
end;

procedure TFNewFunction.ComboTypeClick(Sender: TObject);
begin
 case combotype.itemindex of
  4:
   begin
    LSize.visible:=True;
    LDecimals.visible:=true;
    ESize.visible:=True;
    EDecimals.visible:=True;
    LCharset.visible:=False;
    COmboCharset.visible:=false;
   end;
  6,7,8:
   begin
    LSize.visible:=True;
    LDecimals.visible:=False;
    ESize.visible:=True;
    EDecimals.visible:=False;
    LCharset.visible:=True;
    COmboCharset.visible:=True;
   end;
  else
   begin
    LSize.visible:=False;
    LDecimals.visible:=False;
    ESize.visible:=False;
    EDecimals.visible:=False;
    LCharset.visible:=False;
    COmboCharset.visible:=false;
   end;
 end;
end;

procedure TFNewFunction.DInputTYPEChange(Sender: TField);
var
 astring:string;
 dataset:TDataset;
begin
 dataset:=Sender.DataSet;
 Dataset.fieldbyname('SIZE').readonly:=False;
 Dataset.fieldbyname('DECIMALS').readonly:=False;
 Dataset.fieldbyname('CHARSET').readonly:=False;
 if Not Dataset.fieldbyname('type').isnull then
 begin
  astring:=Uppercase(Dataset.fieldbyname('Type').asstring);
  if ((astring='SMALLINT') or (astring='INTEGER')
     or (astring='FLOAT') or (astring='DOUBLE PRECISION')
     or (astring='DATE') or (astring='TIME') or
      (astring='TIMESTAMP')) then
  begin
   if (Not (Dataset.state in dseditmodes)) then
    Dataset.edit;
   Dataset.fieldbyname('CHARSET').asvariant:=null;
   Dataset.fieldbyname('SIZE').asvariant:=null;
   Dataset.fieldbyname('DECIMALS').asvariant:=null;
   Dataset.fieldbyname('SIZE').readonly:=True;
   Dataset.fieldbyname('DECIMALS').readonly:=True;
   Dataset.fieldbyname('CHARSET').readonly:=True;
  end
  else
  if ((astring='CHAR') or (astring='CSTRING') or (astring='VARCHAR')) then
  begin
   if (Not (Dataset.state in dseditmodes)) then
    Dataset.edit;
   if Dataset.fieldbyname('charset').isnull then
    Dataset.fieldbyname('Charset').asvariant:='WIN1252';
   if Dataset.fieldbyname('size').isnull then
    Dataset.fieldbyname('size').value:=20;
   Dataset.fieldbyname('DECIMALS').asvariant:=null;
   Dataset.fieldbyname('DECIMALS').readonly:=True;
  end
  else
  if (astring='NUMERIC') then
  begin
   if (Not (Dataset.state in dseditmodes)) then
    Dataset.edit;
   if Dataset.fieldbyname('size').isnull then
    Dataset.fieldbyname('size').value:=18;
   if Dataset.fieldbyname('decimals').isnull then
    Dataset.fieldbyname('decimals').value:=2;
   Dataset.fieldbyname('Charset').asvariant:=null;
   Dataset.fieldbyname('Charset').readonly:=True;
  end
  else
  begin
   if (Not (Dataset.state in dseditmodes)) then
    Dataset.edit;
   Dataset.fieldbyname('SIZE').asvariant:=NULL;
   Dataset.fieldbyname('CHARSET').asvariant:=NULL;
   Dataset.fieldbyname('DECIMALS').asvariant:=NULL;
   Dataset.fieldbyname('SIZE').readonly:=True;
   Dataset.fieldbyname('DECIMALS').readonly:=True;
   Dataset.fieldbyname('CHARSET').readonly:=True;
  end;
 end
 else
 begin
  if (Not (Dataset.state in dseditmodes)) then
   Dataset.edit;
  Dataset.fieldbyname('SIZE').asvariant:=NULL;
  Dataset.fieldbyname('CHARSET').asvariant:=NULL;
  Dataset.fieldbyname('DECIMALS').asvariant:=NULL;
 end;
end;

procedure TFNewFunction.DInputAfterScroll(DataSet: TDataSet);
begin
 if (Dataset.eof and Dataset.bof) then
 begin
  Dataset.fieldbyname('Type').readonly:=False;
  Dataset.fieldbyname('Size').readonly:=True;
  Dataset.fieldbyname('Decimals').readonly:=True;
  Dataset.fieldbyname('Charset').readonly:=True;
 end
 else
 begin
  DInputTypeChange(DInputType)
 end;
end;

procedure TFNewFunction.FormCreate(Sender: TObject);
begin
 DInput.CreateDataSet;
end;

end.
