{*******************************************************}
{                                                       }
{       IBAccess                                        }
{       Create and alter Stored dialog                  }
{                                                       }
{       Author:                                         }
{       Toni Martir                                     }
{       http://www.ibaccess.org                         }
{       e-mail: toni@pala.com                           }
{                                                       }
{       OpenSource license (MPL)                        }
{                                                       }
{*******************************************************}

unit newstored;

interface

uses Classes, QForms, QControls, QStdCtrls,
  QButtons, IBSQL, IBDatabase,SysUtils,
  QActnList, QMenus, Db,QDialogs,variants,
  ibacconst,ibacutils, DBClient, QTypes, QGrids, QDBGrids,
  QExtCtrls, QComCtrls;

type
  TFNewStored = class(TForm)
    Trans: TIBTransaction;
    IBSQL1: TIBSQL;
    MainMenu1: TMainMenu;
    MenuItem1: TMenuItem;
    Compile1: TMenuItem;
    ActionList1: TActionList;
    ACompile: TAction;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    DInput: TClientdataset;
    DInputNAME: TStringField;
    DInputTYPE: TStringField;
    SInput: TDataSource;
    Panel2: TPanel;
    Panel3: TPanel;
    Panel4: TPanel;
    GridInput: TDBGrid;
    Label1: TLabel;
    Panel5: TPanel;
    GridOutput: TDBGrid;
    Label2: TLabel;
    DInputCHARSET: TStringField;
    DInputSIZE: TIntegerField;
    DInputDECIMALS: TIntegerField;
    Doutput: TClientdataset;
    DoutputName: TStringField;
    DoutPutType: TStringField;
    DOutputSize: TIntegerField;
    DOutPutdecimals: TIntegerField;
    DOutputcharset: TStringField;
    SOutput: TDataSource;
    IBSQL2: TIBSQL;
    AExecute: TAction;
    Compile2: TMenuItem;
    ToolButton2: TToolButton;
    Procedit: TMemo;
    Panel6: TPanel;
    Label4: TLabel;
    Eprocedurename: TEdit;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ProcEditChange(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure EprocedurenameChange(Sender: TObject);
    procedure DInputTYPEChange(Sender: TField);
    procedure DInputNAMEChange(Sender: TField);
    procedure DInputAfterScroll(DataSet: TDataSet);
    procedure DInputBeforeInsert(DataSet: TDataSet);
    procedure DInputAfterPost(DataSet: TDataSet);
    procedure DInputAfterCancel(DataSet: TDataSet);
    procedure ACompileExecute(Sender: TObject);
    procedure DInputAfterEdit(DataSet: TDataSet);
    procedure DInputAfterInsert(DataSet: TDataSet);
    procedure FormCreate(Sender: TObject);
    procedure fetchparamsint(dia:TFFetching);
    procedure AExecuteExecute(Sender: TObject);
  private
    { Private declarations }
    fforcreate:boolean;
    fbase:TIBDatabase;
    procedure SetBase(Value:TIBDatabase);
    procedure SetForCreate(Value:boolean);
    function buildparams(dataset:TDataset):string;
    function Buildfieldsource(dataset:TDataset):string;
    function COnstructSQL:string;
  public
    { Public declarations }
    dbform:TForm;
    isdestroying:boolean;
    procedure fetchparams;
    property base:TIBDatabase read fbase write Setbase;
    property forcreate:boolean read fforcreate write setforcreate default false;
  end;

implementation

uses databaseform, ibutil2,ibexecsql, mainform;

{$R *.xfm}

procedure TFNewStored.SetForCreate(Value:boolean);
begin
 FForcreate:=Value;
 if FForcreate then
 begin
  EProcedurename.readonly:=false;
//  EProcedurename.color:=clwindow;
  Eprocedurename.text:='NEWPROCEDURE';
 end
 else
  Activecontrol:=ProcEdit;
end;

procedure TFNewStored.Setbase(value: TIBDatabase);
var
 list:TStringlist;
begin
 fbase:=value;
 if value=nil then
  exit;
 list:=TStringlist.create;
 try
  list.add('SMALLINT');
  list.add('INTEGER');
  list.add('FLOAT');
  list.add('DOUBLE PRECISION');
  list.add('NUMERIC');
  list.add('DATE');
  if base.sqldialect=3 then
  begin
   list.add('TIME');
   list.add('TIMESTAMP');
  end;
  list.add('CHAR');
  list.add('VARCHAR');
  list.add('BLOB');
  gridinput.columns[1].PickList.assign(list);
  gridOutput.columns[1].PickList.assign(list);
  list.clear;
  list.add('NONE');
  list.add('OCTECTS');
  list.add('ASCII');
  list.add('UNICODE_FSS');
  list.add('SJIS_0208');
  list.add('EUCJ_0208');
  list.add('DOS437');
  list.add('DOS850');
  list.add('DOS865');
  list.add('ISO8859_1');
  list.add('DOS852');
  list.add('DOS857');
  list.add('DOS860');
  list.add('DOS861');
  list.add('DOS863');
  list.add('CYRL');
  list.add('WIN1250');
  list.add('WIN1251');
  list.add('WIN1252');
  list.add('WIN1253');
  list.add('WIN1254');
  list.add('NEXT');
  list.add('KSC_5601');
  list.add('BIG_5');
  list.add('GB_2312');
  gridinput.columns[4].PickList.assign(list);
  gridoutput.columns[4].PickList.assign(list);
 finally
  list.free;
 end;
end;

procedure TFNewStored.FormClose(Sender: TObject; var Action: TCloseAction);
begin
 action:=cafree;
 isdestroying:=True;
 if fmain.DWindows.Locate('WINDOW',integer(Self),[]) then
  fmain.DWindows.delete;
end;

procedure TFNewStored.ProcEditChange(Sender: TObject);
begin
 ACompile.enabled:=True;
 AExecute.enabled:=false;
end;

procedure TFNewStored.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
var
 res:TModalresult;
begin
 if ACompile.Enabled then
 begin
  res:=MessageDlg(Format(SrpSaveProc,[EProcedurename.text]),mtWarning,[mbYes,mbNo,mbcancel],0);
  if res=mrcancel then
   canclose:=False
  else
   if res=mrYes then
    ACompile.execute;
  end;
end;

procedure TFNewStored.EprocedurenameChange(Sender: TObject);
begin
 ACompile.enabled:=true;
 AExecute.enabled:=false;
end;

procedure TFNewStored.DInputTYPEChange(Sender: TField);
var
 astring:string;
 dataset:TDataset;
begin
 dataset:=Sender.DataSet;
 Dataset.fieldbyname('SIZE').readonly:=False;
 Dataset.fieldbyname('DECIMALS').readonly:=False;
 Dataset.fieldbyname('CHARSET').readonly:=False;
 if Not Dataset.fieldbyname('type').isnull then
 begin
  astring:=Uppercase(Dataset.fieldbyname('Type').asstring);
  if ((astring='SMALLINT') or (astring='INTEGER')
     or (astring='FLOAT') or (astring='DOUBLE PRECISION')
     or (astring='DATE') or (astring='TIME') or
      (astring='TIMESTAMP') or (astring='BLOB')) then
  begin
   if (Not (Dataset.state in dseditmodes)) then
    Dataset.edit;
   Dataset.fieldbyname('CHARSET').asvariant:=null;
   Dataset.fieldbyname('SIZE').asvariant:=null;
   Dataset.fieldbyname('DECIMALS').asvariant:=null;
   Dataset.fieldbyname('SIZE').readonly:=True;
   Dataset.fieldbyname('DECIMALS').readonly:=True;
   Dataset.fieldbyname('CHARSET').readonly:=True;
  end
  else
  if ((astring='CHAR') or (astring='VARCHAR')) then
  begin
   if (Not (Dataset.state in dseditmodes)) then
    Dataset.edit;
   if Dataset.fieldbyname('charset').isnull then
    Dataset.fieldbyname('Charset').asvariant:='WIN1252';
   if Dataset.fieldbyname('size').isnull then
    Dataset.fieldbyname('size').value:=20;
   Dataset.fieldbyname('DECIMALS').asvariant:=null;
   Dataset.fieldbyname('DECIMALS').readonly:=True;
  end
  else
  if (astring='NUMERIC') then
  begin
   if (Not (Dataset.state in dseditmodes)) then
    Dataset.edit;
   if Dataset.fieldbyname('size').isnull then
    Dataset.fieldbyname('size').value:=18;
   if Dataset.fieldbyname('decimals').isnull then
    Dataset.fieldbyname('decimals').value:=2;
   Dataset.fieldbyname('Charset').asvariant:=null;
   Dataset.fieldbyname('Charset').readonly:=True;
  end
  else
  begin
   if (Not (Dataset.state in dseditmodes)) then
    Dataset.edit;
   Dataset.fieldbyname('SIZE').asvariant:=NULL;
   Dataset.fieldbyname('CHARSET').asvariant:=NULL;
   Dataset.fieldbyname('DECIMALS').asvariant:=NULL;
   Dataset.fieldbyname('SIZE').readonly:=True;
   Dataset.fieldbyname('DECIMALS').readonly:=True;
   Dataset.fieldbyname('CHARSET').readonly:=True;
  end;
 end
 else
 begin
  if (Not (Dataset.state in dseditmodes)) then
   Dataset.edit;
  Dataset.fieldbyname('SIZE').asvariant:=NULL;
  Dataset.fieldbyname('CHARSET').asvariant:=NULL;
  Dataset.fieldbyname('DECIMALS').asvariant:=NULL;
 end;
end;

procedure TFNewStored.DInputNAMEChange(Sender: TField);
var
 dataset:TDataset;
begin
 dataset:=Sender.dataset;
 if Dataset.fieldbyname('Name').isnull then
 begin
  Dataset.fieldbyname('Type').readonly:=False;
  Dataset.fieldbyname('Type').asvariant:=null;
  Dataset.fieldbyname('Type').readonly:=True;
 end
 else
 begin
  if length(TRIM(Dataset.fieldbyname('Name').AsString))>0 then
  begin
   Dataset.fieldbyname('Type').readonly:=False;
  end;
 end;
end;

procedure TFNewStored.DInputAfterScroll(DataSet: TDataSet);
begin
 if (Dataset.eof and Dataset.bof) then
 begin
  Dataset.fieldbyname('Type').readonly:=True;
  Dataset.fieldbyname('Size').readonly:=True;
  Dataset.fieldbyname('Decimals').readonly:=True;
  Dataset.fieldbyname('Charset').readonly:=True;
 end
 else
 begin
  if Dataset=Dinput then
   DInputTypeChange(DInputType)
  else
   DInputTypeChange(DOutputType);
 end;
end;

procedure TFNewStored.DInputBeforeInsert(DataSet: TDataSet);
begin
 Dataset.fieldbyname('Type').readonly:=True;
 Dataset.fieldbyname('Size').readonly:=True;
 Dataset.fieldbyname('Decimals').readonly:=True;
 Dataset.fieldbyname('Charset').readonly:=True;
end;

procedure TFNewStored.DInputAfterPost(DataSet: TDataSet);
begin
 Dataset.fieldbyname('Type').readonly:=True;
 Dataset.fieldbyname('Size').readonly:=True;
 Dataset.fieldbyname('Decimals').readonly:=True;
 Dataset.fieldbyname('Charset').readonly:=True;
end;

procedure TFNewStored.DInputAfterCancel(DataSet: TDataSet);
begin
 Dataset.fieldbyname('Type').readonly:=True;
 Dataset.fieldbyname('Size').readonly:=True;
 Dataset.fieldbyname('Decimals').readonly:=True;
 Dataset.fieldbyname('Charset').readonly:=True;
end;


function TFNewStored.Buildfieldsource(dataset:TDataset):string;
var
 astring:string;
begin
 Result:=dataset.fieldbyname('NAME').AsString+' ';
 if length(dataset.fieldbyname('TYPE').AsString)<0 then
  Raise exception.create(SRpMissingFieldType);
 astring:=dataset.fieldbyname('TYPE').AsString;
 if ((astring='SMALLINT') or (astring='INTEGER') or (astring='FLOAT') or
  (astring='DOUBLE PRECISION') or (astring='DATE') or (astring='TIME') or
  (astring='TIMESTAMP') or (astring='BLOB')) then
 begin
  Result:=Result+astring;
 end
 else
 begin
  if ((astring='CHAR') or (astring='VARCHAR')) then
  begin
   Result:=Result+astring;
   if Not dataset.fieldbyname('SIZE').isnull then
   begin
    Result:=Result+'('+dataset.fieldbyname('SIZE').asstring+
     ')';
   end;
  end
  else
  begin
   if (astring='NUMERIC') then
   begin
    Result:=Result+astring;
    if dataset.fieldbyname('SIZE').isnull then
     Raise Exception.create(SRpSizeMustBeSpecified);
    if dataset.fieldbyname('DECIMALS').isnull then
     Raise Exception.create(SRpDecimalsMustBeSpecified);
    Result:=Result+'('+dataset.fieldbyname('SIZE').asstring+
     ','+dataset.fieldbyname('DECIMALS').asstring+')';
   end
   else
   begin
    Raise Exception.Create(Format(SRpUnsuportedType,[astring]));
   end;
  end;
 end;
end;

function TFNewStored.buildparams(dataset:TDataset):string;
var
 amark:TBookmark;
begin
 Result:='';
 if (dataset.eof and dataset.bof) then
  exit;
 dataset.disablecontrols;
 try
  amark:=dataset.getbookmark;
  try
   Result:='(';
   dataset.first;
   While not dataset.eof do
   begin
    if Result='(' then
     Result:=Result+Buildfieldsource(dataset)
    else
     Result:=Result+','+Buildfieldsource(dataset);
    dataset.next;
   end;
   Result:=Result+')';
   dataset.GotoBookmark(amark);
  finally
   Dataset.freebookmark(amark);
  end;
 finally
  dataset.enablecontrols;
 end;
end;

procedure TFNewStored.DInputAfterEdit(DataSet: TDataSet);
begin
 ACompile.enabled:=True;
 AExecute.enabled:=false;
end;

procedure TFNewStored.DInputAfterInsert(DataSet: TDataSet);
begin
 ACompile.enabled:=True;
 AExecute.enabled:=False;
end;

procedure TFNewStored.FormCreate(Sender: TObject);
begin
 DInput.createdataset;
 DOutput.createdataset;
 //
 ProcEdit.text:='AS'+#13+'BEGIN'+#13+' SUSPEND;'+#13+
   'END';
end;

procedure TFNewStored.ACompileExecute(Sender: TObject);
var
 params:string;
 aresult:string;
 astring:string;
 doalter:boolean;
 dform:TFDatabase;
 index:integer;
 adeli:String;
begin
 adeli:='';
 if base.sqldialect=3 then
  adeli:='"'; DInput.checkbrowsemode;
 Doutput.checkbrowsemode;
 // Build the source for parameters
 params:=Buildparams(DInput);
 aresult:=Buildparams(DOutput);

 Trans.defaultdatabase:=base;
 IBSQL1.Database:=base;
 Trans.StartTransaction;
 try
  // Search the stored
  IBSQL1.SQL.text:='SELECT RDB$PROCEDURE_NAME FROM RDB$PROCEDURES WHERE '+
   ' RDB$PROCEDURE_NAME='+QuotedStr(Eprocedurename.text);
  doalter:=false;
  IBSQL1.Execquery;
  try
   If Not IBSQL1.EOF then
   begin
    if fforcreate then
     if mrYEs<>Messagedlg(SRpProcExists,mtWarning,[mbYes,mbNo,mbcancel],0) then
      Raise EAbort.create('');
    doalter:=true;
   end;
  finally
   IBSQL1.close;
  end;
  if doalter then
   astring:='ALTER PROCEDURE '+adeli+EProcedurename.text+adeli
  else
   astring:='CREATE PROCEDURE '+adeli+EProcedurename.text+adeli;
  astring:=astring+' '+params;
  if length(aresult)>0 then
  astring:=astring+' '+'RETURNS '+aresult+' ';
  astring:=astring+ProcEdit.Text;
  IBSQL1.SQL.TExt:=astring;
  IBSQL1.ExecQuery;
  if fforcreate then
  begin
   dform:=TFDatabase(dbform);
   index:=dform.procedureforms.indexofobject(self);
   if index>=0 then
   begin
    dform.procedureforms.strings[index]:=EprocedureName.text;
    caption:=SRpProcedure+':'+EProcedureName.text+' - '+base.databasename;
   end;
  end;
  fforcreate:=false;
  EProcedurename.ReadOnly:=True;
//  EProcedurename.color:=clinfobk;
  Acompile.enabled:=false;
  AExecute.enabled:=True;
  Trans.Commit;
 except
  Trans.Rollback;
  raise;
 end;
end;

procedure InsertParam(dataset:TDataset;aname,atype:string;
 asize,adecimals:variant;acharset:string);
begin
 dataset.append;
 try
  dataset.FieldByName('NAME').Value:=aname;
  dataset.FieldByName('TYPE').Value:=atype;
  if Not (asize=null) then
   dataset.FieldByName('SIZE').Value:=asize;
  if Not (adecimals=null) then
   dataset.FieldByName('DECIMALS').Value:=adecimals;
  if Length(acharset)>0 then
   dataset.FieldByName('CHARSET').Value:=acharset;
  dataset.post;
 except
  dataset.cancel;
  raise;
 end;
end;

procedure TFNewStored.fetchparams;
begin
 DInput.disablecontrols;
 DOutput.disablecontrols;
 try
  ExecuteActionProgres(fetchparamsint,true,nil);
 finally
  DInput.enablecontrols;
  DOutput.enablecontrols;
 end;
end;

procedure TFNewStored.fetchparamsint(dia:TFFetching);
var
 dataset:TDataset;
 acharset,atype:string;
 asize,adecimals:variant;
begin
 Trans.defaultdatabase:=base;
 IBSQL1.Database:=base;
 IBSQL2.database:=base;
 DInput.close;
 Doutput.close;
 Dinput.open;
 DOutput.open;
 Trans.StartTransaction;
 try

  IBSQL1.SQL.text:='SELECT RDB$PROCEDURE_SOURCE FROM RDB$PROCEDURES '+
   'WHERE '+' RDB$PROCEDURE_NAME='+Quotedstr(Eprocedurename.text);
  IBSQL1.ExecQuery;
  try
   dia.IncFetch(1);
   if Not IBSQL1.Eof then
   begin
    ProcEdit.text:='AS'+#13+Trim(IBSQL1.Fields[0].AsString);
   end;
  finally
   IBSQL1.Close;
  end;
  IBSQL1.SQL.text:='SELECT P.RDB$PARAMETER_NAME,P.RDB$PARAMETER_TYPE,'+
   'F.RDB$FIELD_LENGTH,F.RDB$FIELD_SCALE,F.RDB$FIELD_TYPE,'+
   'F.RDB$CHARACTER_SET_ID,F.RDB$CHARACTER_LENGTH'+
   ' FROM RDB$PROCEDURE_PARAMETERS P '+
   ' LEFT OUTER JOIN RDB$FIELDS F ON F.RDB$FIELD_NAME='+
   'P.RDB$FIELD_SOURCE '+
   ' WHERE '+
   ' RDB$PROCEDURE_NAME='+Quotedstr(Eprocedurename.text)+
   ' ORDER BY RDB$PARAMETER_TYPE,RDB$PARAMETER_NUMBER';
  IBSQL1.ExecQuery;
  try
   dia.IncFetch(1);
   IBSQL2.sql.text:='SELECT RDB$CHARACTER_SET_NAME FROM'+
    ' RDB$CHARACTER_SETS WHERE '+
    ' RDB$CHARACTER_SET_ID=:charset';
   While Not IBSQL1.Eof do
   begin
    acharset:='';
    atype:='';
    asize:=-1;
    adecimals:=-1;
    if IBSQL1.FieldByName('RDB$PARAMETER_TYPE').Value=0 then
     dataset:=DInput
    else
     dataset:=Doutput;
    // The datatype
    atype:=ibfieldtypetosourcescale(IBSQL1.fieldbyname('RDB$FIELD_TYPE').AsInteger,
     IBSQL1.fieldbyname('RDB$FIELD_LENGTH').AsInteger,
     IBSQL1.fieldbyname('RDB$FIELD_SCALE').AsInteger,
     asize,adecimals,base.sqldialect);
    if atype='NUMERIC' then
    begin
     adecimals:=-IBSQL1.FieldByName('RDB$FIELD_SCALE').asinteger;
    end
    else
    begin
     if ((atype='CHAR') or (atype='VARCHAR')) then
     begin
      asize:=IBSQL1.FieldByName('RDB$CHARACTER_LENGTH').asinteger;
     end;
    end;
    if Not IBSQL1.FieldByName('RDB$CHARACTER_SET_ID').Isnull then
    begin
     if Not IBSQL2.Prepared then
     begin
      IBSQL2.prepare;
     end;
     IBSQL2.Params.ByName('charset').Value:=IBSQL1.FieldByName('RDB$CHARACTER_SET_ID').Value;
     IBSQL2.ExecQuery;
     try
      dia.IncFetch(1);
      if Not IBSQL2.EOF then
       acharset:=IBSQL2.FieldByName('RDB$CHARACTER_SET_NAME').AsString;
     finally
      IBSQL2.close;
     end;
    end;
    Insertparam(dataset,IBSQL1.FieldByName('RDB$PARAMETER_NAME').AsString,
     atype,asize,adecimals,acharset);
    IBSQL1.Next;
    dia.IncFetch(1);
   end;
  finally
   IBSQL1.Close;
  end;
 finally
  Trans.rollback;
 end;
end;

function TFNewStored.COnstructSQL:string;
begin
 Result:='SELECT * FROM '+Eprocedurename.text;
 if DInput.RecordCount<1 then
  exit;
 Result:=Result+'(';
 DInput.disablecontrols;
 try
  DInput.First;
  While Not DInput.eof do
  begin
   Result:=Result+':'+DInputNAME.AsString;
   DInput.next;
   if not dinput.eof then
    Result:=Result+',';
  end;
 finally
  DInput.enablecontrols;
 end;
 Result:=Result+')';
end;

procedure TFNewStored.AExecuteExecute(Sender: TObject);
var
 aform:TFDatabase;
 qform:TFExecsql;
begin
 // Executes the stored on Exec SQL Window
 if not assigned(dbform) then
  exit;
 aform:=TFDatabase(dbform);
 aform.ASQL.Execute;
 qform:=aform.FormExecSQL;
 qform.MEdit.text:=ConstructSQL;
 qform.AParams.Execute;
 qform.AExecute.execute;
end;
 end.
