{*******************************************************}
{                                                       }
{       IBAccess                                        }
{       Repair database dialog                          }
{                                                       }
{       Author:                                         }
{       Toni Martir                                     }
{       http://www.ibaccess.org                         }
{       e-mail: toni@pala.com                           }
{                                                       }
{       OpenSource license (MPL)                        }
{                                                       }
{*******************************************************}

unit repairdatabase;

interface

uses
  SysUtils, Classes, QForms, QDialogs,
  diaser, IBServices, ibman, rpibdb, Db,
  ibacconst, QMask, dbclient, QExtCtrls, QControls, QGrids, QDBGrids,
  QStdCtrls, QButtons;

type
  TFRepair = class(TFService)
    Label4: TLabel;
    IbMan1: TIbManager;
    IBVal: TIBValidationService;
    RadioOp: TRadioGroup;
    GRestore: TGroupBox;
    RadioService: TRadioButton;
    RadioCommandline: TRadioButton;
    GOptions: TGroupBox;
    CheckIgnorechecksums: TCheckBox;
    CheckKillShadows: TCheckBox;
    CheckValidateRecord: TCheckBox;
    CheckReadOnly: TCheckBox;
    DTrans: TCLientdataset;
    DTransID: TIntegerField;
    DTransMultidatabase: TBooleanField;
    DTransHostsite: TStringField;
    DTransRemoteSite: TStringField;
    DTransRemotedatabsePath: TStringField;
    DTransState: TStringField;
    DTransAdvise: TStringField;
    DTransAction: TStringField;
    DBGrid1: TDBGrid;
    GLimbo: TGroupBox;
    Radioall: TRadioButton;
    RadioID: TRadioButton;
    LAction: TLabel;
    ComboAction: TComboBox;
    DataSource1: TDataSource;
    ETransID: TMaskEdit;
    procedure BOKClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure RadioOpClick(Sender: TObject);
    procedure RadioServiceClick(Sender: TObject);
    procedure RadioallClick(Sender: TObject);
  private
    { Private declarations }
    procedure ExpandDialog;
    procedure Refreshvisibles;
  public
    { Public declarations }
    role:string;
  end;


procedure RepairIBdatabase(var username,password,server,database:string;var protocol:TRpIBXProtocol;dbconf:TRpDBIBconf);


implementation

uses ibproces;

{$R *.xfm}

procedure RepairIBdatabase(var username,password,server,database:string;var protocol:TRpIBXProtocol;dbconf:TRpDBIBconf);
var
 dia:TFRepair;
begin
 dia:=TFRepair.create(Application);
 try
  dia.Eusername.text:=username;
  dia.EPassword.text:=password;
  dia.EServer.text:=server;
  dia.dbconf:=dbconf;
  dia.EServerChange(dia);
  dia.EBase.text:=Database;
  dia.comboprotocol.itemindex:=integer(protocol);
  dia.Radiocommandline.checked:=ibman.globalusecommandline;
  dia.showmodal;
  if dia.executed then
  begin
   username:=dia.USername;
   password:=dia.Password;
   database:=dia.database;
   server:=dia.Server;
   protocol:=dia.protocol;
  end;
 finally
  dia.free;
 end;
end;

procedure TFRepair.BOKClick(Sender: TObject);
const
 transactionstates:array [LimboState..UnknownState] of string=('Limbo','Commit','Rollback','Unknown');
 transactionadvises:array [CommitAdvise..UnknownAdvise] of string=('Commit','Rollback','Unknown');
 transactionactions:array [CommitAction..RollbackAction] of string=('Commit','Rollback');
//  TTransactionState = (LimboState, CommitState, RollbackState, UnknownState);
//  TTransactionAdvise = (CommitAdvise, RollbackAdvise, UnknownAdvise);
//  TTransactionAction = (CommitAction, RollbackAction);
var
 tranid:integer;
 i:integer;
begin
  inherited;
 // Validate the database
 if RadioCommandline.checked then
 begin
  ibman1.serverpath:=ibman.globalserverpath;
  Ibman1.filename:=BuildIBDatabaseName((Length(Trim(EServer.text))>0),TRpIBXProtocol(ComboProtocol.ItemIndex),EServer.Text,EBase.text);
  ibMan1.RepairOptions:=[];
  ibman1.username:=Eusername.text;
  ibman1.password:=EPassword.text;
  case Radioop.ItemIndex of
   0:
    begin
     IBMan1.RepairOptions:=IBMan1.Repairoptions+[reValidate];
     if CheckReadOnly.checked then
      IBMan1.RepairOptions:=IBman1.Repairoptions+[reReadOnly];
     if CheckValidaterecord.checked then
      IBMan1.RepairOptions:=IBMan1.Repairoptions+[reValidaterecords];
     if CheckIgnorechecksums.Checked then
      IBMan1.RepairOptions:=IBMan1.Repairoptions+[reIgnorechecksums];
     if CheckKillShadows.Checked then
      IBMan1.RepairOptions:=IBMan1.Repairoptions+[reKillShadows];
     ibMan1.Repair;
    end;
   1:
    begin
     IBMan1.Sweep;
    end;
   2:
    begin
     IBMan1.RepairOptions:=IBMan1.Repairoptions+[reMendDatabase];
     ibMan1.Repair;
    end;
   3:
    begin
     IBMan1.RepairOptions:=IBMan1.Repairoptions+[relimbotrans];
     ibMan1.Repair;
   end;
   else
   begin
    Ibman1.limboAction:=TlimboAction(ComboAction.ItemIndex);
    if Not RadioAll.checked then
    begin
     try
      tranid:=StrToInt(ETransID.text);
     except
      Raise Exception.Create(SRpValidTransId);
     end;
     IbMan1.RepairLimbotransaction(tranid);
    end
    else
     IbMan1.RepairLimbotransactions;
   end;
  end;
 end
 else
 begin
  IBVal.Params.clear;
  IBVal.params.add('user_name='+EUsername.text);
  IBVal.params.add('password='+EPassword.text);
  IBVal.ServerName:=Trim(EServer.Text);
  IBVal.databasename:=EBase.text;
  if Length(IBVal.ServerName)=0 then
  begin
   IBVal.protocol:=Local;
  end
  else
  begin
   IBVal.protocol:=TProtocol(ComboProtocol.itemindex);
  end;
  IBVal.options:=[];
  case Radioop.ItemIndex of
   0:
    begin
     IBVal.Options:=IBVal.options+[ValidateDB];
     if CheckReadOnly.checked then
      IBVal.Options:=IBVal.options+[CheckDB];
     if CheckValidaterecord.checked then
      IBVal.Options:=IBVal.options+[ValidateFull];
     if CheckIgnorechecksums.Checked then
      IBVal.Options:=IBVal.options+[Ignorechecksum];
     if CheckKillShadows.Checked then
      IBVal.Options:=IBVal.options+[KillShadows];
    end;
   1:
    begin
     IBVal.Options:=IBVal.options+[SweepDB];
    end;
   2:
    begin
     IBVal.Options:=IBVal.options+[MendDB];
     IBVal.Options:=IBVal.options+[Ignorechecksum];
    end;
   3:
    begin
     IBVal.Options:=IBVal.options+[LimboTransactions];
    end;
   else
   begin
    IBVal.Options:=IBVal.options+[LimboTransactions];
    case ComboAction.itemindex of
     0:
      IBVal.GlobalAction:=RollbackGlobal;
     1:
      IBVal.GlobalAction:=CommitGlobal;
     else
     begin
      IBVal.GlobalAction:=RecoverTwoPhaseGlobal;
     end;
    end;
   end;
  end;
  IBvAL.Attach;
  try
   IBvAL.ServiceStart;
   case Radioop.itemindex of
    3:
     begin
      IBVal.FetchLimboTransactionInfo
     end;
    4:
     begin
      IBVal.FetchLimboTransactionInfo;
      IBVal.FixLimboTransactionErrors;
      ibproces.COntrolIBService(IBVal);
     end;
    else
    begin
     ibproces.COntrolIBService(IBVal);
    end;
   end;
  finally
   try
    IBVal.Detach;
   except
   end;
  end;
  if RadioOp.Itemindex=3 then
  begin
   if IBVal.LimboTransactionInfoCount<1 then
   begin
    ShowMessage(SRpNoTransactionsinlimbo);
   end
   else
   begin
    // Fill limbo transactions
    DTrans.Active:=False;
    DTrans.Createdataset;
    for i:=0 to IBVal.LimboTransactionInfoCount-1 do
    begin
     Dtrans.Append;
     try
      DTransID.Value:=IBVal.LimboTransactionInfo[i].ID;
      DTransMultidatabase.Value:=IBVal.LimboTransactionInfo[i].Multidatabase;
      DTransHostsite.Value:=IBVal.LimboTransactionInfo[i].HOstSite;
      DTransRemotesite.Value:=IBVal.LimboTransactionInfo[i].RemoteSite;
      DTransRemotedatabsePath.Value:=IBVal.LimboTransactionInfo[i].Remotedatabasepath;
      DTransState.value:=transactionstates[IBVal.LimboTransactionInfo[i].State];
      DTransAdvise.value:=transactionAdvises[IBVal.LimboTransactionInfo[i].Advise];
      DTransAction.value:=transactionActions[IBVal.LimboTransactionInfo[i].Action];
      DTrans.Post;
     except
      Dtrans.cancel;
      raise;
     end;
    end;
    ExpandDialog;
   end;
  end;
 end;
end;

procedure TFRepair.FormCreate(Sender: TObject);
begin
  inherited;
 RadioOp.Itemindex:=0;
 ComboAction.itemindex:=0;
end;

procedure TFRepair.RadioOpClick(Sender: TObject);
begin
  inherited;
 RefreshVisibles;
end;

procedure TFRepair.RadioServiceClick(Sender: TObject);
begin
  inherited;
 Refreshvisibles;
end;

procedure TFRepair.Refreshvisibles;
begin
 GOptions.visible:=RadioOp.Itemindex=0;
 Glimbo.Visible:=((RadioOp.Itemindex=4) AND RadioCommandline.Checked);
 LAction.visible:=RadioOp.Itemindex=4;
 ComboAction.visible:=RadioOp.Itemindex=4;
end;

procedure TFRepair.ExpandDialog;
begin
 Height:=DBGrid1.Top+DBGrid1.Height+5;
 DBGrid1.Visible:=True;
end;

procedure TFRepair.RadioallClick(Sender: TObject);
begin
  inherited;
 ETransID.Visible:=(Not Radioall.Checked);
end;

end.
